@@ -1,7 +1,7 @@
 /* machines_periph.h: various machine-specific peripherals
-   Copyright (c) 2011 Philip Kendall
+   Copyright (c) 2011-2016 Philip Kendall
 
-   $Id: machines_periph.h 4775 2012-11-26 23:03:36Z sbaldovi $
+   $Id: machines_periph.h 5677 2016-07-09 13:58:02Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #ifndef FUSE_MACHINES_PERIPH_H
 #define FUSE_MACHINES_PERIPH_H
 
-void machines_periph_init( void );
+void machines_periph_register_startup( void );
 
 void machines_periph_48( void );
 void machines_periph_128( void );
