@@ -1,8 +1,8 @@
 /* disciple.c: Routines for handling the DISCiPLE interface
-   Copyright (c) 1999-2011 Stuart Brady, Fredrick Meunier, Philip Kendall,
-   Dmitry Sanarin, Darren Salt
+   Copyright (c) 1999-2016 Stuart Brady, Fredrick Meunier, Philip Kendall,
+   Dmitry Sanarin, Darren Salt, Gergely Szasz
 
-   $Id: disciple.c 4906 2013-03-10 11:21:37Z fredm $
+   $Id: disciple.c 5677 2016-07-09 13:58:02Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,20 +33,20 @@
 #include <string.h>
 
 #include "compat.h"
+#include "debugger/debugger.h"
 #include "disciple.h"
+#include "infrastructure/startup_manager.h"
 #include "machine.h"
 #include "module.h"
 #include "peripherals/printer.h"
 #include "settings.h"
 #include "ui/ui.h"
+#include "ui/uimedia.h"
 #include "unittests/unittests.h"
 #include "utils.h"
 #include "wd_fdc.h"
 #include "options.h"	/* needed for get combo options */
 
-#define DISK_TRY_MERGE(heads) ( option_enumerate_diskoptions_disk_try_merge() == 2 || \
-				( option_enumerate_diskoptions_disk_try_merge() == 1 && heads == 1 ) )
-
 /* Two 8 KiB memory chunks accessible by the Z80 when /ROMCS is low */
 /* One 8 KiB chunk of ROM, one 8 KiB chunk of RAM */
 /* TODO: add support for 16 KiB ROM images. */
@@ -67,34 +67,58 @@ int disciple_inhibited;
 int disciple_available = 0;
 int disciple_active = 0;
 
-static int disciple_index_pulse;
-
-static int index_event;
-
-#define DISCIPLE_NUM_DRIVES 2
-
 static wd_fdc *disciple_fdc;
-static wd_fdc_drive disciple_drives[ DISCIPLE_NUM_DRIVES ];
+static fdd_t disciple_drives[ DISCIPLE_NUM_DRIVES ];
+static ui_media_drive_info_t disciple_ui_drives[ DISCIPLE_NUM_DRIVES ];
 
 static libspectrum_byte *disciple_ram;
 static int memory_allocated = 0;
 
 static void disciple_reset( int hard_reset );
 static void disciple_memory_map( void );
-static void disciple_event_index( libspectrum_dword last_tstates, int type,
-				  void *user_data );
 static void disciple_activate( void );
+static void disciple_enabled_snapshot( libspectrum_snap *snap );
+static void disciple_from_snapshot( libspectrum_snap *snap );
+static void disciple_to_snapshot( libspectrum_snap *snap );
+
+/* WD1770 registers */
+static libspectrum_byte disciple_sr_read( libspectrum_word port, libspectrum_byte *attached );
+static void disciple_cr_write( libspectrum_word port, libspectrum_byte b );
+static libspectrum_byte disciple_tr_read( libspectrum_word port, libspectrum_byte *attached );
+static void disciple_tr_write( libspectrum_word port, libspectrum_byte b );
+static libspectrum_byte disciple_sec_read( libspectrum_word port, libspectrum_byte *attached );
+static void disciple_sec_write( libspectrum_word port, libspectrum_byte b );
+static libspectrum_byte disciple_dr_read( libspectrum_word port, libspectrum_byte *attached );
+static void disciple_dr_write( libspectrum_word port, libspectrum_byte b );
+
+static libspectrum_byte disciple_joy_read( libspectrum_word port, libspectrum_byte *attached );
+static void disciple_cn_write( libspectrum_word port, libspectrum_byte b );
+static void disciple_net_write( libspectrum_word port, libspectrum_byte b);
+static libspectrum_byte disciple_boot_read( libspectrum_word port, libspectrum_byte *attached );
+static void disciple_boot_write( libspectrum_word port, libspectrum_byte b );
+static libspectrum_byte disciple_patch_read( libspectrum_word port, libspectrum_byte *attached );
+static void disciple_patch_write( libspectrum_word port, libspectrum_byte b );
+static void disciple_printer_write( libspectrum_word port, libspectrum_byte b );
+
+/* 8KB ROM */
+#define ROM_SIZE 0x2000
+/* 8KB RAM */
+#define RAM_SIZE 0x2000
 
 static module_info_t disciple_module_info = {
 
-  disciple_reset,
-  disciple_memory_map,
-  NULL,
-  NULL,
-  NULL,
+  /* .reset = */ disciple_reset,
+  /* .romcs = */ disciple_memory_map,
+  /* .snapshot_enabled = */ disciple_enabled_snapshot,
+  /* .snapshot_from = */ disciple_from_snapshot,
+  /* .snapshot_to = */ disciple_to_snapshot,
 
 };
 
+/* Debugger events */
+static const char * const event_type_string = "disciple";
+static int page_event, unpage_event;
+
 static libspectrum_byte disciple_control_register;
 
 void
@@ -103,6 +127,7 @@ disciple_page( void )
   disciple_active = 1;
   machine_current->ram.romcs = 1;
   machine_current->memory_map();
+  debugger_event( page_event );
 }
 
 void
@@ -111,6 +136,7 @@ disciple_unpage( void )
   disciple_active = 0;
   machine_current->ram.romcs = 0;
   machine_current->memory_map();
+  debugger_event( unpage_event );
 }
 
 void
@@ -160,39 +186,36 @@ static const periph_port_t disciple_ports[] = {
 };
 
 static const periph_t disciple_periph = {
-  &settings_current.disciple,
-  disciple_ports,
-  1,
-  disciple_activate
+  /* .option = */ &settings_current.disciple,
+  /* .ports = */ disciple_ports,
+  /* .hard_reset = */ 1,
+  /* .activate = */ disciple_activate,
 };
 
-void
-disciple_init( void )
+static int
+disciple_init( void *context )
 {
   int i;
-  wd_fdc_drive *d;
+  fdd_t *d;
 
   disciple_fdc = wd_fdc_alloc_fdc( WD1770, 0, WD_FLAG_NONE );
 
   for( i = 0; i < DISCIPLE_NUM_DRIVES; i++ ) {
     d = &disciple_drives[ i ];
-    fdd_init( &d->fdd, FDD_SHUGART, NULL, 0 );
+    fdd_init( d, FDD_SHUGART, NULL, 0 );
     d->disk.flag = DISK_FLAG_NONE;
   }
 
   disciple_fdc->current_drive = &disciple_drives[ 0 ];
-  fdd_select( &disciple_drives[ 0 ].fdd, 1 );
+  fdd_select( &disciple_drives[ 0 ], 1 );
   disciple_fdc->dden = 1;
   disciple_fdc->set_intrq = NULL;
   disciple_fdc->reset_intrq = NULL;
   disciple_fdc->set_datarq = NULL;
   disciple_fdc->reset_datarq = NULL;
-  disciple_fdc->iface = NULL;
-
-  index_event = event_register( disciple_event_index, "DISCiPLE index" );
 
   module_register( &disciple_module_info );
-  
+
   disciple_memory_source_rom = memory_source_register( "DISCiPLE ROM" );
   disciple_memory_source_ram = memory_source_register( "DISCiPLE RAM" );
 
@@ -209,20 +232,46 @@ disciple_init( void )
   }
 
   periph_register( PERIPH_TYPE_DISCIPLE, &disciple_periph );
+
+  for( i = 0; i < DISCIPLE_NUM_DRIVES; i++ ) {
+    disciple_ui_drives[ i ].fdd = &disciple_drives[ i ];
+    ui_media_drive_register( &disciple_ui_drives[ i ] );
+  }
+
+  periph_register_paging_events( event_type_string, &page_event,
+                                 &unpage_event );
+
+  return 0;
+}
+
+static void
+disciple_end( void )
+{
+  disciple_available = 0;
+  libspectrum_free( disciple_fdc );
+}
+
+void
+disciple_register_startup( void )
+{
+  startup_manager_module dependencies[] = {
+    STARTUP_MANAGER_MODULE_DEBUGGER,
+    STARTUP_MANAGER_MODULE_MEMORY,
+    STARTUP_MANAGER_MODULE_SETUID,
+  };
+  startup_manager_register( STARTUP_MANAGER_MODULE_DISCIPLE, dependencies,
+                            ARRAY_SIZE( dependencies ), disciple_init, NULL,
+                            disciple_end );
 }
 
 static void
 disciple_reset( int hard_reset )
 {
   int i;
-  wd_fdc_drive *d;
-  const fdd_params_t *dt;
 
   disciple_active = 0;
   disciple_available = 0;
 
-  event_remove_type( index_event );
-
   if( !periph_is_active( PERIPH_TYPE_DISCIPLE ) ) {
     return;
   }
@@ -230,14 +279,17 @@ disciple_reset( int hard_reset )
   /* TODO: add support for 16 KiB ROM images. */
   if( machine_load_rom_bank( disciple_memory_map_romcs_rom, 0,
 			     settings_current.rom_disciple,
-			     settings_default.rom_disciple, 0x2000 ) ) {
+			     settings_default.rom_disciple, ROM_SIZE ) ) {
     settings_current.disciple = 0;
     periph_activate_type( PERIPH_TYPE_DISCIPLE, 0 );
     return;
   }
 
-  for( i = 0; i < MEMORY_PAGES_IN_8K; i++ )
-    disciple_memory_map_romcs_ram[ i ].page = &disciple_ram[ i * MEMORY_PAGE_SIZE ];
+  for( i = 0; i < MEMORY_PAGES_IN_8K; i++ ) {
+    struct memory_page *page = &disciple_memory_map_romcs_ram[ i ];
+    page->page = &disciple_ram[ i * MEMORY_PAGE_SIZE ];
+    page->offset = i * MEMORY_PAGE_SIZE;
+  }
 
   machine_current->ram.romcs = 1;
 
@@ -246,125 +298,90 @@ disciple_reset( int hard_reset )
 
   disciple_available = 1;
   disciple_active = 1;
-  disciple_index_pulse = 0;
 
   disciple_memswap = 0;
   /* TODO: add support for 16 KiB ROM images. */
   /* disciple_rombank = 0; */
 
   if( hard_reset )
-    memset( disciple_ram, 0, 0x2000 );
+    memset( disciple_ram, 0, RAM_SIZE );
 
   wd_fdc_master_reset( disciple_fdc );
 
   for( i = 0; i < DISCIPLE_NUM_DRIVES; i++ ) {
-    d = &disciple_drives[ i ];
-
-    d->index_pulse = 0;
-    d->index_interrupt = 0;
+    ui_media_drive_update_menus( &disciple_ui_drives[ i ],
+                                 UI_MEDIA_DRIVE_UPDATE_ALL );
   }
 
-  /* We can eject disks only if they are currently present */
-  dt = &fdd_params[ option_enumerate_diskoptions_drive_disciple1_type() + 1 ];
-  fdd_init( &disciple_drives[ DISCIPLE_DRIVE_1 ].fdd, FDD_SHUGART, dt, 1 );
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1, dt->enabled );
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_EJECT,
-		    disciple_drives[ DISCIPLE_DRIVE_1 ].fdd.loaded );
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_FLIP_SET,
-		    !disciple_drives[ DISCIPLE_DRIVE_1 ].fdd.upsidedown );
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_WP_SET,
-		    !disciple_drives[ DISCIPLE_DRIVE_1 ].fdd.wrprot );
-
-
-  dt = &fdd_params[ option_enumerate_diskoptions_drive_disciple2_type() ];
-  fdd_init( &disciple_drives[ DISCIPLE_DRIVE_2 ].fdd, dt->enabled ? FDD_SHUGART : FDD_TYPE_NONE, dt, 1 );
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2, dt->enabled );
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_EJECT,
-		    disciple_drives[ DISCIPLE_DRIVE_2 ].fdd.loaded );
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_FLIP_SET,
-		    !disciple_drives[ DISCIPLE_DRIVE_2 ].fdd.upsidedown );
-  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_WP_SET,
-		    !disciple_drives[ DISCIPLE_DRIVE_2 ].fdd.wrprot );
-
-
   disciple_fdc->current_drive = &disciple_drives[ 0 ];
-  fdd_select( &disciple_drives[ 0 ].fdd, 1 );
+  fdd_select( &disciple_drives[ 0 ], 1 );
   machine_current->memory_map();
-  disciple_event_index( 0, index_event, NULL );
 
-  ui_statusbar_update( UI_STATUSBAR_ITEM_DISK, UI_STATUSBAR_STATE_INACTIVE );
 }
 
-void
+static void
 disciple_inhibit( void )
 {
   /* TODO: check how this affects the hardware */
   disciple_inhibited = 1;
 }
 
-void
-disciple_end( void )
+static libspectrum_byte
+disciple_sr_read( libspectrum_word port GCC_UNUSED, libspectrum_byte *attached )
 {
-  disciple_available = 0;
-  free( disciple_fdc );
-}
-
-libspectrum_byte
-disciple_sr_read( libspectrum_word port GCC_UNUSED, int *attached )
-{
-  *attached = 1;
+  *attached = 0xff;
   return wd_fdc_sr_read( disciple_fdc );
 }
 
-void
+static void
 disciple_cr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
   wd_fdc_cr_write( disciple_fdc, b );
 }
 
-libspectrum_byte
-disciple_tr_read( libspectrum_word port GCC_UNUSED, int *attached )
+static libspectrum_byte
+disciple_tr_read( libspectrum_word port GCC_UNUSED, libspectrum_byte *attached )
 {
-  *attached = 1;
+  *attached = 0xff;
   return wd_fdc_tr_read( disciple_fdc );
 }
 
-void
+static void
 disciple_tr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
   wd_fdc_tr_write( disciple_fdc, b );
 }
 
-libspectrum_byte
-disciple_sec_read( libspectrum_word port GCC_UNUSED, int *attached )
+static libspectrum_byte
+disciple_sec_read( libspectrum_word port GCC_UNUSED, libspectrum_byte *attached )
 {
-  *attached = 1;
+  *attached = 0xff;
   return wd_fdc_sec_read( disciple_fdc );
 }
 
-void
+static void
 disciple_sec_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
   wd_fdc_sec_write( disciple_fdc, b );
 }
 
-libspectrum_byte
-disciple_dr_read( libspectrum_word port GCC_UNUSED, int *attached )
+static libspectrum_byte
+disciple_dr_read( libspectrum_word port GCC_UNUSED, libspectrum_byte *attached )
 {
-  *attached = 1;
+  *attached = 0xff;
   return wd_fdc_dr_read( disciple_fdc );
 }
 
-void
+static void
 disciple_dr_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
   wd_fdc_dr_write( disciple_fdc, b );
 }
 
-libspectrum_byte
-disciple_joy_read( libspectrum_word port GCC_UNUSED, int *attached )
+static libspectrum_byte
+disciple_joy_read( libspectrum_word port GCC_UNUSED, libspectrum_byte *attached )
 {
-  *attached = 1;
+  *attached = 0xff; /* TODO: check this */
 
   /* bit 6 - printer busy */
   if( !settings_current.printer )
@@ -373,7 +390,7 @@ disciple_joy_read( libspectrum_word port GCC_UNUSED, int *attached )
   return 0xff;   /* never busy */
 }
 
-void
+static void
 disciple_cn_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
 {
   int drive, side;
@@ -385,14 +402,14 @@ disciple_cn_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
   side = ( b & 0x02 ) ? 1 : 0;
 
   for( i = 0; i < DISCIPLE_NUM_DRIVES; i++ ) {
-    fdd_set_head( &disciple_drives[ i ].fdd, side );
-    fdd_select( &disciple_drives[ i ].fdd, drive == i );
+    fdd_set_head( &disciple_drives[ i ], side );
+    fdd_select( &disciple_drives[ i ], drive == i );
   }
 
   if( disciple_fdc->current_drive != &disciple_drives[ drive ] ) {
-    if( disciple_fdc->current_drive->fdd.motoron ) {
+    if( disciple_fdc->current_drive->motoron ) {
       for (i = 0; i < DISCIPLE_NUM_DRIVES; i++ ) {
-        fdd_motoron( &disciple_drives[ i ].fdd, drive == i );
+        fdd_motoron( &disciple_drives[ i ], drive == i );
       }
     }
     disciple_fdc->current_drive = &disciple_drives[ drive ];
@@ -407,23 +424,23 @@ disciple_cn_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
     disciple_inhibit();
 }
 
-void
+static void
 disciple_net_write( libspectrum_word port GCC_UNUSED,
 		    libspectrum_byte b GCC_UNUSED )
 {
   /* TODO: implement network emulation */
 }
 
-libspectrum_byte
+static libspectrum_byte
 disciple_boot_read( libspectrum_word port GCC_UNUSED,
-		    int *attached GCC_UNUSED )
+		    libspectrum_byte *attached GCC_UNUSED )
 {
   disciple_memswap = 0;
   machine_current->memory_map();
   return 0;
 }
 
-void
+static void
 disciple_boot_write( libspectrum_word port GCC_UNUSED,
 		     libspectrum_byte b GCC_UNUSED )
 {
@@ -431,22 +448,22 @@ disciple_boot_write( libspectrum_word port GCC_UNUSED,
   machine_current->memory_map();
 }
 
-libspectrum_byte
+static libspectrum_byte
 disciple_patch_read( libspectrum_word port GCC_UNUSED,
-		     int *attached GCC_UNUSED )
+		     libspectrum_byte *attached GCC_UNUSED )
 {
   disciple_page();
   return 0;
 }
 
-void
+static void
 disciple_patch_write( libspectrum_word port GCC_UNUSED,
 		    libspectrum_byte b GCC_UNUSED )
 {
   disciple_unpage();
 }
 
-void
+static void
 disciple_printer_write( libspectrum_word port, libspectrum_byte b )
 {
   printer_parallel_write( port, b );
@@ -456,258 +473,26 @@ int
 disciple_disk_insert( disciple_drive_number which, const char *filename,
 		      int autoload )
 {
-  int error;
-  wd_fdc_drive *d;
-  const fdd_params_t *dt;
-
   if( which >= DISCIPLE_NUM_DRIVES ) {
     ui_error( UI_ERROR_ERROR, "disciple_disk_insert: unknown drive %d",
 	      which );
     fuse_abort();
   }
 
-  d = &disciple_drives[ which ];
-
-  /* Eject any disk already in the drive */
-  if( d->fdd.loaded ) {
-    /* Abort the insert if we want to keep the current disk */
-    if( disciple_disk_eject( which ) ) return 0;
-  }
-
-  if( filename ) {
-    error = disk_open( &d->disk, filename, 0, DISK_TRY_MERGE( d->fdd.fdd_heads ) );
-    if( error != DISK_OK ) {
-      ui_error( UI_ERROR_ERROR, "Failed to open disk image: %s",
-				disk_strerror( error ) );
-      return 1;
-    }
-  } else {
-    switch( which ) {
-    case 0:
-      dt = &fdd_params[ option_enumerate_diskoptions_drive_disciple1_type() + 1 ];	/* +1 => there is no 'Disabled' */
-      break;
-    case 1:
-    default:
-      dt = &fdd_params[ option_enumerate_diskoptions_drive_disciple2_type() ];
-      break;
-    }
-    error = disk_new( &d->disk, dt->heads, dt->cylinders, DISK_DENS_AUTO, DISK_UDI );
-    if( error != DISK_OK ) {
-      ui_error( UI_ERROR_ERROR, "Failed to create disk image: %s",
-				disk_strerror( error ) );
-      return 1;
-    }
-  }
-
-  fdd_load( &d->fdd, &d->disk, 0 );
-
-  /* Set the 'eject' item active */
-  switch( which ) {
-  case DISCIPLE_DRIVE_1:
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_EJECT, 1 );
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_FLIP_SET,
-		      !disciple_drives[ DISCIPLE_DRIVE_1 ].fdd.upsidedown );
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_WP_SET,
-		      !disciple_drives[ DISCIPLE_DRIVE_1 ].fdd.wrprot );
-    break;
-  case DISCIPLE_DRIVE_2:
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_EJECT, 1 );
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_FLIP_SET,
-		      !disciple_drives[ DISCIPLE_DRIVE_2 ].fdd.upsidedown );
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_WP_SET,
-		      !disciple_drives[ DISCIPLE_DRIVE_2 ].fdd.wrprot );
-    break;
-  }
-
-  if( filename && autoload ) {
-    /* XXX */
-  }
-
-  return 0;
-}
-
-int
-disciple_disk_eject( disciple_drive_number which )
-{
-  wd_fdc_drive *d;
-
-  if( which >= DISCIPLE_NUM_DRIVES )
-    return 1;
-
-  d = &disciple_drives[ which ];
-
-  if( d->disk.type == DISK_TYPE_NONE )
-    return 0;
-
-  if( d->disk.dirty ) {
-
-    ui_confirm_save_t confirm = ui_confirm_save(
-      "Disk in DISCiPLE drive %c has been modified.\n"
-      "Do you want to save it?",
-      which == DISCIPLE_DRIVE_1 ? '1' : '2'
-    );
-
-    switch( confirm ) {
-
-    case UI_CONFIRM_SAVE_SAVE:
-      if( disciple_disk_save( which, 0 ) ) return 1;	/* first save */
-      break;
-
-    case UI_CONFIRM_SAVE_DONTSAVE: break;
-    case UI_CONFIRM_SAVE_CANCEL: return 1;
-
-    }
-  }
-
-  fdd_unload( &d->fdd );
-  disk_close( &d->disk );
-
-  /* Set the 'eject' item inactive */
-  switch( which ) {
-  case DISCIPLE_DRIVE_1:
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_EJECT, 0 );
-    break;
-  case DISCIPLE_DRIVE_2:
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_EJECT, 0 );
-    break;
-  }
-  return 0;
-}
-
-int
-disciple_disk_save( disciple_drive_number which, int saveas )
-{
-  wd_fdc_drive *d;
-
-  if( which >= DISCIPLE_NUM_DRIVES )
-    return 1;
-
-  d = &disciple_drives[ which ];
-
-  if( d->disk.type == DISK_TYPE_NONE )
-    return 0;
-
-  if( d->disk.filename == NULL ) saveas = 1;
-  if( ui_disciple_disk_write( which, saveas ) ) return 1;
-  d->disk.dirty = 0;
-  return 0;
-}
-
-int
-disciple_disk_flip( disciple_drive_number which, int flip )
-{
-  wd_fdc_drive *d;
-
-  if( which >= DISCIPLE_NUM_DRIVES )
-    return 1;
-
-  d = &disciple_drives[ which ];
-
-  if( !d->fdd.loaded )
-    return 1;
-
-  fdd_flip( &d->fdd, flip );
-
-  /* Update the 'write flip' menu item */
-  switch( which ) {
-  case DISCIPLE_DRIVE_1:
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_FLIP_SET,
-		      !disciple_drives[ DISCIPLE_DRIVE_1 ].fdd.upsidedown );
-    break;
-  case DISCIPLE_DRIVE_2:
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_FLIP_SET,
-		      !disciple_drives[ DISCIPLE_DRIVE_2 ].fdd.upsidedown );
-    break;
-  }
-  return 0;
-}
-
-int
-disciple_disk_writeprotect( disciple_drive_number which, int wrprot )
-{
-  wd_fdc_drive *d;
-
-  if( which >= DISCIPLE_NUM_DRIVES )
-    return 1;
-
-  d = &disciple_drives[ which ];
-
-  if( !d->fdd.loaded )
-    return 1;
-
-  fdd_wrprot( &d->fdd, wrprot );
-
-  /* Update the 'write protect' menu item */
-  switch( which ) {
-  case DISCIPLE_DRIVE_1:
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_WP_SET,
-		      !disciple_drives[ DISCIPLE_DRIVE_1 ].fdd.wrprot );
-    break;
-  case DISCIPLE_DRIVE_2:
-    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_WP_SET,
-		      !disciple_drives[ DISCIPLE_DRIVE_2 ].fdd.wrprot );
-    break;
-  }
-  return 0;
-}
-
-/***TODO most part of the next routine could be move to a common place... */
-int
-disciple_disk_write( disciple_drive_number which, const char *filename )
-{
-  wd_fdc_drive *d = &disciple_drives[ which ];
-  int error;
-
-  d->disk.type = DISK_TYPE_NONE;
-  if( filename == NULL ) filename = d->disk.filename;	/* write over original file */
-  error = disk_write( &d->disk, filename );
-
-  if( error != DISK_OK ) {
-    ui_error( UI_ERROR_ERROR, "couldn't write '%s' file: %s", filename,
-	      disk_strerror( error ) );
-    return 1;
-  }
-  if( d->disk.filename && strcmp( filename, d->disk.filename ) ) {
-    free( d->disk.filename );
-    d->disk.filename = utils_safe_strdup( filename );
-  }
-
-  return 0;
+  return ui_media_drive_insert( &disciple_ui_drives[ which ], filename, autoload );
 }
 
 fdd_t *
 disciple_get_fdd( disciple_drive_number which )
 {
-  return &( disciple_drives[ which ].fdd );
-}
-
-static void
-disciple_event_index( libspectrum_dword last_tstates, int type GCC_UNUSED,
-		      void *user_data GCC_UNUSED )
-{
-  int next_tstates;
-  int i;
-
-  disciple_index_pulse = !disciple_index_pulse;
-  for( i = 0; i < DISCIPLE_NUM_DRIVES; i++ ) {
-    wd_fdc_drive *d = &disciple_drives[ i ];
-
-    d->index_pulse = disciple_index_pulse;
-    if( !disciple_index_pulse && d->index_interrupt ) {
-      wd_fdc_set_intrq( disciple_fdc );
-      d->index_interrupt = 0;
-    }
-  }
-  next_tstates = ( disciple_index_pulse ? 10 : 190 ) *
-    machine_current->timings.processor_speed / 1000;
-  event_add( last_tstates + next_tstates, index_event );
+  return &( disciple_drives[ which ] );
 }
 
 static void
 disciple_activate( void )
 {
   if( !memory_allocated ) {
-    disciple_ram = memory_pool_allocate_persistent( 0x2000, 1 );
+    disciple_ram = memory_pool_allocate_persistent( RAM_SIZE, 1 );
     memory_allocated = 1;
   }
 }
@@ -764,3 +549,153 @@ disciple_unittest( void )
 
   return r;
 }
+
+static int
+ui_drive_is_available( void )
+{
+  return disciple_available;
+}
+
+static const fdd_params_t *
+ui_drive_get_params_1( void )
+{
+  /* +1 => there is no `Disabled' */
+  return &fdd_params[ option_enumerate_diskoptions_drive_disciple1_type() + 1 ];
+}
+
+static const fdd_params_t *
+ui_drive_get_params_2( void )
+{
+  return &fdd_params[ option_enumerate_diskoptions_drive_disciple2_type() ];
+}
+
+static ui_media_drive_info_t disciple_ui_drives[ DISCIPLE_NUM_DRIVES ] = {
+  {
+    /* .name = */ "DISCiPLE Disk 1",
+    /* .controller_index = */ UI_MEDIA_CONTROLLER_DISCIPLE,
+    /* .drive_index = */ DISCIPLE_DRIVE_1,
+    /* .menu_item_parent = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE,
+    /* .menu_item_top = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1,
+    /* .menu_item_eject = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_EJECT,
+    /* .menu_item_flip = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_FLIP_SET,
+    /* .menu_item_wp = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_WP_SET,
+    /* .is_available = */ &ui_drive_is_available,
+    /* .get_params = */ &ui_drive_get_params_1,
+  },
+  {
+    /* .name = */ "DISCiPLE Disk 2",
+    /* .controller_index = */ UI_MEDIA_CONTROLLER_DISCIPLE,
+    /* .drive_index = */ DISCIPLE_DRIVE_2,
+    /* .menu_item_parent = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE,
+    /* .menu_item_top = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2,
+    /* .menu_item_eject = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_EJECT,
+    /* .menu_item_flip = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_FLIP_SET,
+    /* .menu_item_wp = */ UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_2_WP_SET,
+    /* .is_available = */ &ui_drive_is_available,
+    /* .get_params = */ &ui_drive_get_params_2,
+  },
+};
+
+static void
+disciple_enabled_snapshot( libspectrum_snap *snap )
+{
+  if( libspectrum_snap_disciple_active( snap ) )
+    settings_current.disciple = 1;
+}
+
+static void
+disciple_from_snapshot( libspectrum_snap *snap )
+{
+  int i;
+
+  if( !libspectrum_snap_disciple_active( snap ) ) return;
+
+  if( libspectrum_snap_disciple_custom_rom( snap ) &&
+      libspectrum_snap_disciple_rom( snap, 0 ) &&
+      machine_load_rom_bank_from_buffer(
+                             disciple_memory_map_romcs_rom, 0,
+                             libspectrum_snap_disciple_rom( snap, 0 ),
+                             libspectrum_snap_disciple_rom_length( snap, 0 ),
+                             1 ) )
+    return;
+
+  if( libspectrum_snap_disciple_ram( snap, 0 ) ) {
+    for( i = 0; i < MEMORY_PAGES_IN_8K; i++ )
+      memcpy( disciple_memory_map_romcs_ram[ i ].page,
+              libspectrum_snap_disciple_ram( snap, 0 ) + i * MEMORY_PAGE_SIZE,
+              MEMORY_PAGE_SIZE );
+  }
+
+  /* ignore drive count for now, there will be an issue with loading snaps where
+     drives have been disabled
+  libspectrum_snap_disciple_drive_count( snap )
+   */
+
+  /* FIXME: As disciple_inhibited can flip on and off the hardware inhibit
+     button is effectively always pressed
+  libspectrum_snap_disciple_inhibit_button( snap );
+   */
+
+  disciple_fdc->direction = libspectrum_snap_plusd_direction( snap );
+
+  disciple_cr_write ( 0x001b, libspectrum_snap_disciple_status ( snap ) );
+  disciple_tr_write ( 0x005b, libspectrum_snap_disciple_track  ( snap ) );
+  disciple_sec_write( 0x009b, libspectrum_snap_disciple_sector ( snap ) );
+  disciple_dr_write ( 0x00db, libspectrum_snap_disciple_data   ( snap ) );
+  disciple_cn_write ( 0x001f, libspectrum_snap_disciple_control( snap ) );
+  /* FIXME: Set disciple_inhibited based on the value in
+     libspectrum_snap_disciple_control() */
+
+  if( libspectrum_snap_disciple_paged( snap ) ) {
+    disciple_page();
+  } else {
+    disciple_unpage();
+  }
+}
+
+static void
+disciple_to_snapshot( libspectrum_snap *snap )
+{
+  libspectrum_byte *buffer;
+  int drive_count = 0;
+  int i;
+
+  if( !periph_is_active( PERIPH_TYPE_DISCIPLE ) ) return;
+
+  libspectrum_snap_set_disciple_active( snap, 1 );
+
+  /* Always save ROM to snapshot to ensure any loading emulator has the matching
+     image */
+  libspectrum_snap_set_disciple_custom_rom( snap, 1 );
+  libspectrum_snap_set_disciple_rom_length( snap, 0, ROM_SIZE );
+
+  buffer = libspectrum_new( libspectrum_byte, ROM_SIZE );
+
+  for( i = 0; i < MEMORY_PAGES_IN_8K; i++ )
+    memcpy( buffer + i * MEMORY_PAGE_SIZE,
+            disciple_memory_map_romcs_rom[ i ].page, MEMORY_PAGE_SIZE );
+
+  libspectrum_snap_set_disciple_rom( snap, 0, buffer );
+
+  buffer = libspectrum_new( libspectrum_byte, RAM_SIZE );
+
+  for( i = 0; i < MEMORY_PAGES_IN_8K; i++ )
+    memcpy( buffer + i * MEMORY_PAGE_SIZE,
+            disciple_memory_map_romcs_ram[ i ].page, MEMORY_PAGE_SIZE );
+  libspectrum_snap_set_disciple_ram( snap, 0, buffer );
+
+  drive_count++; /* Drive 1 is not removable */
+  if( option_enumerate_diskoptions_drive_disciple2_type() > 0 ) drive_count++;
+  libspectrum_snap_set_disciple_drive_count( snap, drive_count );
+
+  libspectrum_snap_set_disciple_paged ( snap, disciple_active );
+  /* FIXME: As disciple_inhibited can flip on and off the hardware inhibit
+     button is effectively always pressed but should be emulated */
+  libspectrum_snap_set_disciple_inhibit_button( snap, 1 );
+  libspectrum_snap_set_disciple_direction( snap, disciple_fdc->direction );
+  libspectrum_snap_set_disciple_status( snap, disciple_fdc->status_register );
+  libspectrum_snap_set_disciple_track ( snap, disciple_fdc->track_register );
+  libspectrum_snap_set_disciple_sector( snap, disciple_fdc->sector_register );
+  libspectrum_snap_set_disciple_data  ( snap, disciple_fdc->data_register );
+  libspectrum_snap_set_disciple_control( snap, disciple_control_register );
+}
