@@ -1,7 +1,8 @@
 /* upd_fdc.c: NEC floppy disk controller emulation
-   Copyright (c) 2007-2010 Gergely Szasz
+   Copyright (c) 2007-2015 Gergely Szasz
+   Copyright (c) 2015 Stuart Brady
 
-   $Id: upd_fdc.c 4636 2012-01-20 14:07:15Z pak21 $
+   $Id: upd_fdc.c 5434 2016-05-01 04:22:45Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -131,12 +132,12 @@ upd_fdc_master_reset( upd_fdc *f )
   int i;
   for( i = 0; i < 4; i++ )
     if( f->drive[i] != NULL )
-      fdd_select( &f->drive[i]->fdd, i == 0 ? 1 : 0 );
+      fdd_select( f->drive[i], i == 0 ? 1 : 0 );
   f->current_drive = f->drive[0];
 
   f->main_status = UPD_FDC_MAIN_DATAREQ;
   for( i = 0; i < 4; i++ )
-    f->status_register[i] = f->pcn[i] = f->seek[i] = 0;
+    f->status_register[i] = f->pcn[i] = f->seek[i] = f->seek_age[i] = 0;
   f->stp_rate = 16;
   f->hut_time = 240;
   f->hld_time = 254;
@@ -159,9 +160,9 @@ crc_preset( upd_fdc *f )
 }
 
 static void
-crc_add( upd_fdc *f, upd_fdc_drive *d )
+crc_add( upd_fdc *f, fdd_t *d )
 {
-  f->crc = crc_fdc( f->crc, d->fdd.data & 0xff );
+  f->crc = crc_fdc( f->crc, d->data & 0xff );
 }
 
 /* 
@@ -174,58 +175,58 @@ static int
 read_id( upd_fdc *f )
 {
   int i;
-  upd_fdc_drive *d = f->current_drive;
+  fdd_t *d = f->current_drive;
 
   f->status_register[1] &= ~( UPD_FDC_ST1_CRC_ERROR | 
     			      UPD_FDC_ST1_MISSING_AM |
     			      UPD_FDC_ST1_NO_DATA );
   f->id_mark = UPD_FDC_AM_NONE;
   i = f->rev;
-  while( i == f->rev && d->fdd.ready ) {
-    fdd_read_write_data( &d->fdd, FDD_READ ); if( d->fdd.index ) f->rev--;
+  while( i == f->rev && d->ready ) {
+    fdd_read_data( d ); if( d->index ) f->rev--;
     crc_preset( f );
     if( f->mf ) {	/* double density (MFM) */
-      if( d->fdd.data == 0xffa1 ) {
+      if( d->data == 0xffa1 ) {
         crc_add( f, d );
-	fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-	if( d->fdd.index ) f->rev--;
-	if( d->fdd.data != 0xffa1 )
+	fdd_read_data( d ); crc_add( f, d );
+	if( d->index ) f->rev--;
+	if( d->data != 0xffa1 )
 	  continue;
-	fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-	if( d->fdd.index ) f->rev--;
-	if( d->fdd.data != 0xffa1 )
+	fdd_read_data( d ); crc_add( f, d );
+	if( d->index ) f->rev--;
+	if( d->data != 0xffa1 )
 	  continue;
       } else {		/* no 0xa1 with missing clock... */
 	continue;
       }
     }
-    fdd_read_write_data( &d->fdd, FDD_READ );
-    if( d->fdd.index ) f->rev--;
+    fdd_read_data( d );
+    if( d->index ) f->rev--;
     if( f->mf ) {	/* double density (MFM) */
-      if( d->fdd.data != 0x00fe )
+      if( d->data != 0x00fe )
 	continue;
     } else {		/* single density (FM) */
-      if( d->fdd.data != 0xfffe )
+      if( d->data != 0xfffe )
 	continue;
     }
     crc_add( f, d );
-    fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-    if( d->fdd.index ) f->rev--;
-    f->id_track = d->fdd.data;
-    fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-    if( d->fdd.index ) f->rev--;
-    f->id_head = d->fdd.data;
-    fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-    if( d->fdd.index ) f->rev--;
-    f->id_sector = d->fdd.data;
-    fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-    if( d->fdd.index ) f->rev--;
-    f->id_length = d->fdd.data > MAX_SIZE_CODE ? MAX_SIZE_CODE : d->fdd.data;
+    fdd_read_data( d ); crc_add( f, d );
+    if( d->index ) f->rev--;
+    f->id_track = d->data;
+    fdd_read_data( d ); crc_add( f, d );
+    if( d->index ) f->rev--;
+    f->id_head = d->data;
+    fdd_read_data( d ); crc_add( f, d );
+    if( d->index ) f->rev--;
+    f->id_sector = d->data;
+    fdd_read_data( d ); crc_add( f, d );
+    if( d->index ) f->rev--;
+    f->id_length = d->data > MAX_SIZE_CODE ? MAX_SIZE_CODE : d->data;
     f->sector_length = 0x80 << f->id_length; 
-    fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-    if( d->fdd.index ) f->rev--;
-    fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-    if( d->fdd.index ) f->rev--;
+    fdd_read_data( d ); crc_add( f, d );
+    if( d->index ) f->rev--;
+    fdd_read_data( d ); crc_add( f, d );
+    if( d->index ) f->rev--;
 
     if( f->crc != 0x0000 ) {
       f->status_register[1] |= UPD_FDC_ST1_CRC_ERROR | UPD_FDC_ST1_NO_DATA;
@@ -236,7 +237,7 @@ read_id( upd_fdc *f )
       return 0;		/* found and OK */
     }
   }
-  if(!d->fdd.ready) f->rev = 0;
+  if(!d->ready) f->rev = 0;
   f->status_register[1] |= UPD_FDC_ST1_MISSING_AM | UPD_FDC_ST1_NO_DATA;	/*FIXME _NO_DATA? */
   return 2;		/* not found */
 }
@@ -250,16 +251,16 @@ read_id( upd_fdc *f )
 static int
 read_datamark( upd_fdc *f )
 {
-  upd_fdc_drive *d = f->current_drive;
+  fdd_t *d = f->current_drive;
   int i;
 
   if( f->mf ) {	/* double density (MFM) */
     for( i = 40; i > 0; i-- ) {
-      fdd_read_write_data( &d->fdd, FDD_READ );
-      if( d->fdd.data == 0x4e )		/* read next */
+      fdd_read_data( d );
+      if( d->data == 0x4e )		/* read next */
 	continue;
 
-      if( d->fdd.data == 0x00 )		/* go to PLL sync */
+      if( d->data == 0x00 )		/* go to PLL sync */
 	break;
 
       f->status_register[2] |= UPD_FDC_ST2_MISSING_DM;
@@ -267,40 +268,40 @@ read_datamark( upd_fdc *f )
     } 
     for( ; i > 0; i-- ) {
       crc_preset( f );
-      fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-      if( d->fdd.data == 0x00 )
+      fdd_read_data( d ); crc_add( f, d );
+      if( d->data == 0x00 )
 	continue;
 
-      if( d->fdd.data == 0xffa1 )	/* got to a1 mark */
+      if( d->data == 0xffa1 )	/* got to a1 mark */
 	break;
 
       f->status_register[2] |= UPD_FDC_ST2_MISSING_DM;
       return 1;
     }
-    for( i = d->fdd.data == 0xffa1 ? 2 : 3; i > 0; i-- ) {
-      fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-      if( d->fdd.data != 0xffa1 ) {
+    for( i = d->data == 0xffa1 ? 2 : 3; i > 0; i-- ) {
+      fdd_read_data( d ); crc_add( f, d );
+      if( d->data != 0xffa1 ) {
         f->status_register[2] |= UPD_FDC_ST2_MISSING_DM;
 	return 1;
       }
     } 
-    fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-    if( d->fdd.data < 0x00f8 || d->fdd.data > 0x00fb ) { /* !fb deleted mark */
+    fdd_read_data( d ); crc_add( f, d );
+    if( d->data < 0x00f8 || d->data > 0x00fb ) { /* !fb deleted mark */
       f->status_register[2] |= UPD_FDC_ST2_MISSING_DM;
       return 1;
     }
-    if( d->fdd.data != 0x00fb )
+    if( d->data != 0x00fb )
       f->ddam = 1;
     else
       f->ddam = 0;
     return 0;
   } else {		/* SD -> FM */
     for( i = 30; i > 0; i-- ) {
-      fdd_read_write_data( &d->fdd, FDD_READ );
-      if( d->fdd.data == 0xff )		/* read next */
+      fdd_read_data( d );
+      if( d->data == 0xff )		/* read next */
 	continue;
 
-      if( d->fdd.data == 0x00 )		/* go to PLL sync */
+      if( d->data == 0x00 )		/* go to PLL sync */
 	break;
 
       f->status_register[2] |= UPD_FDC_ST2_MISSING_DM;
@@ -308,24 +309,24 @@ read_datamark( upd_fdc *f )
     } 
     for( ; i > 0; i-- ) {
       crc_preset( f );
-      fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-      if( d->fdd.data == 0x00 )
+      fdd_read_data( d ); crc_add( f, d );
+      if( d->data == 0x00 )
 	continue;
 
-      if( d->fdd.data >= 0xfff8 && d->fdd.data <= 0xfffb )	/* !fb deleted mark */
+      if( d->data >= 0xfff8 && d->data <= 0xfffb )	/* !fb deleted mark */
 	break;
 
       f->status_register[2] |= UPD_FDC_ST2_MISSING_DM;
       return 1;
     }
     if( i == 0 ) {
-      fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-      if( d->fdd.data < 0xfff8 || d->fdd.data > 0xfffb ) {	/* !fb deleted mark */
+      fdd_read_data( d ); crc_add( f, d );
+      if( d->data < 0xfff8 || d->data > 0xfffb ) {	/* !fb deleted mark */
         f->status_register[2] |= UPD_FDC_ST2_MISSING_DM;
 	return 1;
       }
     }
-    if( d->fdd.data != 0x00fb )
+    if( d->data != 0x00fb )
       f->ddam = 1;
     else
       f->ddam = 0;
@@ -381,8 +382,7 @@ upd_fdc_alloc_fdc( upd_type_t type, upd_clock_t clock )
 {
   int i;
   
-  upd_fdc *f = malloc( sizeof( *f ) );
-  if( !f ) return NULL;
+  upd_fdc *f = libspectrum_new( upd_fdc, 1 );
 
   f->type = type != UPD765B ? UPD765A : UPD765B;
   f->clock = clock != UPD_CLOCK_4MHZ ? UPD_CLOCK_8MHZ : UPD_CLOCK_4MHZ;
@@ -418,50 +418,87 @@ cmd_result( upd_fdc *f )
 }
 
 static void
-seek_step( upd_fdc *f )
+seek_step( upd_fdc *f, int start )
 {
-  int i;
-  upd_fdc_drive *d;
-  
-  for( i = 0; i < 4; i++ ) {			/* look after all disk... */
-    if( f->main_status & ( 1 << i ) ) {		/* this drive in seek state */
-      d = f->drive[i];
-      if( f->pcn[i] == f->ncn[i] &&
-          f->seek[i] == 2 && !d->fdd.tr00 ) {	/* recalibrate fail */
-        f->seek[i] = 5;				/* abnormal termination */
-	f->intrq = UPD_INTRQ_SEEK;
-	f->status_register[0] |= UPD_FDC_ST0_EQUIP_CHECK;
-	f->main_status &= ~( 1 << i );
-	continue;
-      }
-      if( f->pcn[i] == f->ncn[i] || 
-    	  ( f->seek[i] == 2 && d->fdd.tr00 ) ) {	/* end of seek */
-	if( f->seek[i] == 2 )			/* recalibrate */
-	  f->pcn[i] = 0; 
-        f->seek[i] = 4;				/* normal termination */
-	f->intrq = UPD_INTRQ_SEEK;
-	f->main_status &= ~( 1 << i );
-	continue;
-      }
+  int i, j;
+  fdd_t *d;
 
-      if( !d->fdd.ready ) {
-	if( f->seek[i] == 2 )			/* recalibrate */
-	  f->pcn[i] = f->rec[i] - ( 77 - f->pcn[i] ); 	/* restore PCN */
-        f->seek[i] = 6;				/* drive not ready termination */
-	f->intrq = UPD_INTRQ_READY;		/* doesn't matter */
-	f->main_status &= ~( 1 << i );
-      } else if( f->pcn[i] != f->ncn[i] ) {	/**FIXME if d->tr00 == 1 ??? */
-        fdd_step( &d->fdd, f->pcn[i] > f->ncn[i] ? 
-			    FDD_STEP_OUT : FDD_STEP_IN );
-        f->pcn[i] += f->pcn[i] > f->ncn[i] ? -1 : 1;
-      }
+  if( start ) {
+    i = f->us;
+
+    /* Drive already in seek state? */
+    if( f->main_status & ( 1 << i ) ) return;   	
+
+    /* Mark seek mode for fdd. It will be cleared by Sense Interrupt command */
+    f->main_status |= 1 << i;
+  } else {
+
+    /* Get drive in seek state that has completed the positioning */
+    i=0;
+    for( j = 1; j < 4; j++) {
+      if( f->seek_age[j] > f->seek_age[i] )
+        i = j;
+    }
+
+    if( f->seek[i] == 0 || f->seek[i] >= 4 ) {
+      return;
     }
   }
-  if( f->main_status & 0x0f ) {		/* there is at least one active seek */
-    event_add_with_data( tstates + f->stp_rate * 
-			 machine_current->timings.processor_speed / 1000,
-			 fdc_event, f );
+
+  d = f->drive[i];
+
+  /* There is need to seek? */
+  if( f->pcn[i] == f->ncn[i] &&
+      f->seek[i] == 2 && !d->tr00 ) {	/* recalibrate fail */
+    f->seek[i] = 5;				/* abnormal termination */
+    f->seek_age[i] = 0;
+    f->intrq = UPD_INTRQ_SEEK;
+    f->status_register[0] |= UPD_FDC_ST0_EQUIP_CHECK;
+    f->main_status &= ~( 1 << i );
+    return;
   }
+
+  /* There is need to seek? */
+  if( f->pcn[i] == f->ncn[i] || 
+    ( f->seek[i] == 2 && d->tr00 ) ) {	/* correct position */
+    if( f->seek[i] == 2 )			/* recalibrate */
+      f->pcn[i] = 0; 
+    f->seek[i] = 4;				/* normal termination */
+    f->seek_age[i] = 0;
+    f->intrq = UPD_INTRQ_SEEK;
+    f->main_status &= ~( 1 << i );
+    return;
+  }
+
+  /* Drive not ready */
+  if( !d->ready ) {
+    if( f->seek[i] == 2 )			/* recalibrate */
+      f->pcn[i] = f->rec[i] - ( 77 - f->pcn[i] ); 	/* restore PCN */
+    f->seek[i] = 6;				/* drive not ready termination */
+    f->seek_age[i] = 0;
+    f->intrq = UPD_INTRQ_READY;		/* doesn't matter */
+    f->main_status &= ~( 1 << i );
+    return;
+  }
+
+  /* Send step */
+  if( f->pcn[i] != f->ncn[i] ) {	/**FIXME if d->tr00 == 1 ??? */
+    fdd_step( d, f->pcn[i] > f->ncn[i] ? 
+                        FDD_STEP_OUT : FDD_STEP_IN );
+    f->pcn[i] += f->pcn[i] > f->ncn[i] ? -1 : 1;
+
+    /* Update age for active seek operations */
+    for( j = 0; j < 4; j++) {
+      if( f->seek_age[j] > 0 ) f->seek_age[j]++;
+    }
+    f->seek_age[i] = 1;
+
+    /* wait step completion */
+    event_add_with_data( tstates + f->stp_rate * 
+                         machine_current->timings.processor_speed / 1000,
+                         fdc_event, f );
+  }
+
   return;
 }
 
@@ -660,7 +697,7 @@ static void
 start_write_data( upd_fdc *f )
 {
   int i;
-  upd_fdc_drive *d = f->current_drive;
+  fdd_t *d = f->current_drive;
 
 multi_track_next:
   if( f->first_rw || f->read_id ||
@@ -697,24 +734,24 @@ multi_track_next:
     }
 
     for( i = 11; i > 0; i-- )	/* "delay" 11 GAP byte */
-      fdd_read_write_data( &d->fdd, FDD_READ );
+      fdd_read_data( d );
     if( f->mf )					/* MFM */
       for( i = 11; i > 0; i-- )	/* "delay" another 11 GAP byte */
-	fdd_read_write_data( &d->fdd, FDD_READ );
+	fdd_read_data( d );
 
-    d->fdd.data = 0x00;
+    d->data = 0x00;
     for( i = f->mf ? 12 : 6; i > 0; i-- )	/* write 6/12 zero */
-      fdd_read_write_data( &d->fdd, FDD_WRITE );
+      fdd_write_data( d );
     crc_preset( f );
     if( f->mf ) {				/* MFM */
-      d->fdd.data = 0xffa1;
+      d->data = 0xffa1;
       for( i = 3; i > 0; i-- ) {		/* write 3 0xa1 with clock mark */
-	fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+	fdd_write_data( d ); crc_add( f, d );
       }
     }
-    d->fdd.data = ( f->del_data ? 0x00f8 : 0x00fb ) |
+    d->data = ( f->del_data ? 0x00f8 : 0x00fb ) |
     			( f->mf ? 0x0000 : 0xff00 );	/* write data mark */
-    fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+    fdd_write_data( d ); crc_add( f, d );
   } else {
     f->data_register[1]++;		/* next track */
     f->data_register[3] = 1;		/* first sector */
@@ -750,33 +787,33 @@ static void
 start_write_id( upd_fdc *f )
 {
   int i;
-  upd_fdc_drive *d = f->current_drive;
+  fdd_t *d = f->current_drive;
 
-  d->fdd.data = f->mf ? 0x4e : 0xff;
+  d->data = f->mf ? 0x4e : 0xff;
   for( i = 40; i > 0; i-- )	/* write 40 GAP byte */
-    fdd_read_write_data( &d->fdd, FDD_WRITE );
+    fdd_write_data( d );
   if( f->mf )					/* MFM */
     for( i = 40; i > 0; i-- )	/* write another 40 GAP byte */
-      fdd_read_write_data( &d->fdd, FDD_WRITE );
-  d->fdd.data = 0x00;
+      fdd_write_data( d );
+  d->data = 0x00;
   for( i = f->mf ? 12 : 6; i > 0; i-- )	/* write 6/12 zero */
-    fdd_read_write_data( &d->fdd, FDD_WRITE );
+    fdd_write_data( d );
   crc_preset( f );
   if( f->mf ) {				/* MFM */
-    d->fdd.data = 0xffc2;
+    d->data = 0xffc2;
     for( i = 3; i > 0; i-- ) {		/* write 3 0xc2 with clock mark */
-      fdd_read_write_data( &d->fdd, FDD_WRITE );
+      fdd_write_data( d );
     }
   }
-  d->fdd.data = 0x00fc | ( f->mf ? 0x0000 : 0xff00 );	/* write index mark */
-  fdd_read_write_data( &d->fdd, FDD_WRITE );
+  d->data = 0x00fc | ( f->mf ? 0x0000 : 0xff00 );	/* write index mark */
+  fdd_write_data( d );
 
-  d->fdd.data = f->mf ? 0x4e : 0xff;	/* postindex GAP */
+  d->data = f->mf ? 0x4e : 0xff;	/* postindex GAP */
   for( i = 26; i > 0; i-- )	/* write 26 GAP byte */
-    fdd_read_write_data( &d->fdd, FDD_WRITE );
+    fdd_write_data( d );
   if( f->mf )					/* MFM */
     for( i = 24; i > 0; i-- )	/* write another 24 GAP byte */
-      fdd_read_write_data( &d->fdd, FDD_WRITE );
+      fdd_write_data( d );
 
   f->main_status |= UPD_FDC_MAIN_DATAREQ | UPD_FDC_MAIN_DATA_WRITE;
   f->data_offset = 0;
@@ -795,16 +832,16 @@ head_load( upd_fdc *f )
     else if( f->cmd->id == UPD_CMD_READ_ID )
       start_read_id( f );
     else if( f->cmd->id == UPD_CMD_READ_DIAG ) {
-      fdd_wait_index_hole( &f->current_drive->fdd );		/* start reading from index hole */
+      fdd_wait_index_hole( f->current_drive );		/* start reading from index hole */
       start_read_diag( f );
     } else if( f->cmd->id == UPD_CMD_WRITE_DATA )
       start_write_data( f );
     else if( f->cmd->id == UPD_CMD_WRITE_ID ) {
-      fdd_wait_index_hole( &f->current_drive->fdd );		/* start writing from index hole */
+      fdd_wait_index_hole( f->current_drive );		/* start writing from index hole */
       start_write_id( f );
     }
   } else {
-    fdd_head_load( &f->current_drive->fdd, 1 );
+    fdd_head_load( f->current_drive, 1 );
     f->head_load = 1;
     event_add_with_data( tstates + f->hld_time * 
 			 machine_current->timings.processor_speed / 1000,
@@ -826,7 +863,7 @@ upd_fdc_event( libspectrum_dword last_tstates GCC_UNUSED, int event,
   }
 
   if( event == head_event ) {
-    fdd_head_load( &f->current_drive->fdd, 0 );
+    fdd_head_load( f->current_drive, 0 );
     f->head_load = 0;
     return;
   }
@@ -842,18 +879,18 @@ upd_fdc_event( libspectrum_dword last_tstates GCC_UNUSED, int event,
       start_write_data( f );
     }
   } else if( f->main_status & 0x03 ) {		/* seek/recalibrate active */
-    seek_step( f );
+    seek_step( f, 0 );
   } else if( f->cmd->id == UPD_CMD_READ_DATA || f->cmd->id == UPD_CMD_SCAN ) {
     start_read_data( f );
   } else if( f->cmd->id == UPD_CMD_READ_ID ) {
     start_read_id( f );
   } else if( f->cmd->id == UPD_CMD_READ_DIAG ) {
-    fdd_wait_index_hole( &f->current_drive->fdd );		/* start reading from index hole */
+    fdd_wait_index_hole( f->current_drive );		/* start reading from index hole */
     start_read_diag( f );
   } else if( f->cmd->id == UPD_CMD_WRITE_DATA ) {
     start_write_data( f );
   } else if( f->cmd->id == UPD_CMD_WRITE_ID ) {
-    fdd_wait_index_hole( &f->current_drive->fdd );		/* start writing from index hole */
+    fdd_wait_index_hole( f->current_drive );		/* start writing from index hole */
     start_write_id( f );
   }
 
@@ -871,7 +908,7 @@ upd_fdc_read_data( upd_fdc *f )
 {
   libspectrum_byte r;
 
-  upd_fdc_drive *d = f->current_drive;
+  fdd_t *d = f->current_drive;
 
   if( !( f->main_status & UPD_FDC_MAIN_DATAREQ ) || 
       !( f->main_status & UPD_FDC_MAIN_DATA_READ ) )
@@ -879,31 +916,31 @@ upd_fdc_read_data( upd_fdc *f )
 
   if( f->state == UPD_FDC_STATE_EXE ) {		/* READ_DATA/READ_DIAG */
     f->data_offset++;				/* count read bytes */
-    fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d ); /* read a byte */
+    fdd_read_data( d ); crc_add( f, d );	/* read a byte */
 
     /* Speedlock hack */
-    if( f->speedlock > 0 && !d->fdd.do_read_weak ) {	/* do not conflict with fdd weak reads */
-      if( f->data_offset < 64 && d->fdd.data != 0xe5 )
+    if( f->speedlock > 0 && !d->do_read_weak ) {	/* do not conflict with fdd weak reads */
+      if( f->data_offset < 64 && d->data != 0xe5 )
         f->speedlock = 2;		/* W.E.C Le Mans type ... */
       else if( ( f->speedlock > 1 || f->data_offset < 64 ) &&
 	       !( f->data_offset % 29 ) ) {
-	d->fdd.data ^= f->data_offset;	/* mess up data */
+	d->data ^= f->data_offset;	/* mess up data */
 	crc_add( f, d );			/* mess up crc */
       }
     }
     /* EOSpeedlock hack */
 
-    r = d->fdd.data & 0xff;
+    r = d->data & 0xff;
     if( f->data_offset == f->rlen ) {	/* send only rlen byte to host */
       while( f->data_offset < f->sector_length ) {
-	fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
+	fdd_read_data( d ); crc_add( f, d );
 	f->data_offset++;
       }
     }
-    if( ( f->cmd->id == UPD_CMD_READ_DIAG && f->data_offset == f->rlen ) ||
-	( f->cmd->id == UPD_CMD_READ_DATA && f->data_offset == f->sector_length ) ) {	/* read the CRC */
-      fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-      fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
+    if( ( f->cmd->id == UPD_CMD_READ_DIAG || f->cmd->id == UPD_CMD_READ_DATA )
+        && f->data_offset == f->sector_length ) {       /* read the CRC */
+      fdd_read_data( d ); crc_add( f, d );
+      fdd_read_data( d ); crc_add( f, d );
       if( f->crc != 0x000 ) {
 	f->status_register[2] |= UPD_FDC_ST2_DATA_ERROR;
 	f->status_register[1] |= UPD_FDC_ST1_CRC_ERROR;
@@ -967,9 +1004,8 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
 {
   int i, terminated = 0;
   unsigned int u;
-  upd_fdc_drive *d;
+  fdd_t *d;
 
-  
   if( !( f->main_status & UPD_FDC_MAIN_DATAREQ ) || 
       ( f->main_status & UPD_FDC_MAIN_DATA_READ ) )
     return;
@@ -984,67 +1020,67 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
       if( f->data_offset == 4 ) {			/* C, H, R, N done => format track */
 	event_remove_type( timeout_event );
 
-        d->fdd.data = 0x00;
+        d->data = 0x00;
         for( i = f->mf ? 12 : 6; i > 0; i-- )	/* write 6/12 zero */
-          fdd_read_write_data( &d->fdd, FDD_WRITE );
+          fdd_write_data( d );
         crc_preset( f );
         if( f->mf ) {				/* MFM */
-          d->fdd.data = 0xffa1;
+          d->data = 0xffa1;
           for( i = 3; i > 0; i-- ) {		/* write 3 0xa1 with clock mark */
-	    fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+	    fdd_write_data( d ); crc_add( f, d );
           }
         }
-        d->fdd.data = 0x00fe | ( f->mf ? 0x0000 : 0xff00 );	/* write id mark */
-        fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+        d->data = 0x00fe | ( f->mf ? 0x0000 : 0xff00 );	/* write id mark */
+        fdd_write_data( d ); crc_add( f, d );
 	for( i = 0; i < 4; i++ ) {
-    	  d->fdd.data =  f->data_register[i + 5];	/* write id fields */
-          fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+    	  d->data =  f->data_register[i + 5];	/* write id fields */
+          fdd_write_data( d ); crc_add( f, d );
 	}
-        d->fdd.data = f->crc >> 8;
-        fdd_read_write_data( &d->fdd, FDD_WRITE );	/* write crc1 */
-        d->fdd.data = f->crc & 0xff;
-        fdd_read_write_data( &d->fdd, FDD_WRITE );	/* write crc2 */
+        d->data = f->crc >> 8;
+        fdd_write_data( d );			/* write crc1 */
+        d->data = f->crc & 0xff;
+        fdd_write_data( d );			/* write crc2 */
 
-        d->fdd.data = f->mf ? 0x4e : 0xff;
+        d->data = f->mf ? 0x4e : 0xff;
         for( i = 11; i > 0; i-- )	/* write 11 GAP byte */
-          fdd_read_write_data( &d->fdd, FDD_WRITE );
+          fdd_write_data( d );
         if( f->mf )					/* MFM */
           for( i = 11; i > 0; i-- )	/* write another 11 GAP byte */
-	    fdd_read_write_data( &d->fdd, FDD_WRITE );
-        d->fdd.data = 0x00;
+	    fdd_write_data( d );
+        d->data = 0x00;
         for( i = f->mf ? 12 : 6; i > 0; i-- )	/* write 6/12 zero */
-          fdd_read_write_data( &d->fdd, FDD_WRITE );
+          fdd_write_data( d );
         crc_preset( f );
         if( f->mf ) {				/* MFM */
-          d->fdd.data = 0xffa1;
+          d->data = 0xffa1;
           for( i = 3; i > 0; i-- ) {		/* write 3 0xa1 with clock mark */
-	    fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+	    fdd_write_data( d ); crc_add( f, d );
           }
         }
-        d->fdd.data = 0x00fb | ( f->mf ? 0x0000 : 0xff00 );	/* write data mark */
-        fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+        d->data = 0x00fb | ( f->mf ? 0x0000 : 0xff00 );	/* write data mark */
+        fdd_write_data( d ); crc_add( f, d );
 	
-    	d->fdd.data =  f->data_register[4];	/* write filler byte */
+    	d->data =  f->data_register[4];	/* write filler byte */
 	for( i = f->rlen; i > 0; i-- ) {
-          fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+          fdd_write_data( d ); crc_add( f, d );
 	}
-        d->fdd.data = f->crc >> 8;
-        fdd_read_write_data( &d->fdd, FDD_WRITE );	/* write crc1 */
-        d->fdd.data = f->crc & 0xff;
-        fdd_read_write_data( &d->fdd, FDD_WRITE );	/* write crc2 */
+        d->data = f->crc >> 8;
+        fdd_write_data( d );			/* write crc1 */
+        d->data = f->crc & 0xff;
+        fdd_write_data( d );			/* write crc2 */
 
-        d->fdd.data = f->mf ? 0x4e : 0xff;
+        d->data = f->mf ? 0x4e : 0xff;
 	for( i = f->data_register[3]; i > 0; i-- ) {	/* GAP */
-          fdd_read_write_data( &d->fdd, FDD_WRITE );
+          fdd_write_data( d );
 	}
         f->data_offset = 0;
 	f->data_register[2]--;		/* prepare next sector */
       }
       if( f->data_register[2] == 0 ) {	/* finish all sector */
 
-        d->fdd.data = f->mf ? 0x4e : 0xff;	/* GAP3 as Intel call this GAP */
-        while( !d->fdd.index )
-          fdd_read_write_data( &d->fdd, FDD_WRITE );
+        d->data = f->mf ? 0x4e : 0xff;	/* GAP3 as Intel call this GAP */
+        while( !d->index )
+          fdd_write_data( d );
 
 
 	f->state = UPD_FDC_STATE_RES;	/* end of execution phase */
@@ -1060,43 +1096,43 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
       return;
     } else if( f->cmd->id == UPD_CMD_WRITE_DATA ) {		/* WRITE DATA */
       f->data_offset++;
-      d->fdd.data = data;
-      fdd_read_write_data( &d->fdd, FDD_WRITE ); crc_add( f, d );
+      d->data = data;
+      fdd_write_data( d ); crc_add( f, d );
     
       if( f->data_offset == f->rlen ) {	/* read only rlen byte from host */
-        d->fdd.data = 0x00;
+        d->data = 0x00;
         while( f->data_offset < f->sector_length ) {	/* fill with 0x00 */
-	  fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
+	  fdd_read_data( d ); crc_add( f, d );
 	  f->data_offset++;
         }
       }
       if( f->data_offset == f->sector_length ) {	/* write the CRC */
-        d->fdd.data = f->crc >> 8;
-        fdd_read_write_data( &d->fdd, FDD_WRITE );	/* write crc1 */
-        d->fdd.data = f->crc & 0xff;
-        fdd_read_write_data( &d->fdd, FDD_WRITE );	/* write crc2 */
+        d->data = f->crc >> 8;
+        fdd_write_data( d );			/* write crc1 */
+        d->data = f->crc & 0xff;
+        fdd_write_data( d );			/* write crc2 */
         f->main_status &= ~UPD_FDC_MAIN_DATAREQ;
 	start_write_data( f );
       }
       return;
     } else {						/* SCAN */
       f->data_offset++;
-      fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-      if( f->data_offset == 0 && d->fdd.data == data )	/* `scan hit' */
+      fdd_read_data( d ); crc_add( f, d );
+      if( f->data_offset == 0 && d->data == data )	/* `scan hit' */
         f->status_register[2] |= UPD_FDC_ST2_SCAN_HIT;
 
-      if( d->fdd.data != data )				/* `scan _not_ hit' */
+      if( d->data != data )				/* `scan _not_ hit' */
         f->status_register[2] &= ~UPD_FDC_ST2_SCAN_HIT;
 
-      if( ( f->scan == UPD_SCAN_EQ && d->fdd.data != data ) ||	/* scan not satisfied */
-          ( f->scan == UPD_SCAN_LO && d->fdd.data > data ) ||
-          ( f->scan == UPD_SCAN_HI && d->fdd.data < data ) ) {
+      if( ( f->scan == UPD_SCAN_EQ && d->data != data ) ||	/* scan not satisfied */
+          ( f->scan == UPD_SCAN_LO && d->data > data ) ||
+          ( f->scan == UPD_SCAN_HI && d->data < data ) ) {
         f->status_register[2] |= UPD_FDC_ST2_SCAN_NOT_SAT;
       }
     
       if( f->data_offset == f->sector_length ) {	/* read the CRC */
-	fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
-	fdd_read_write_data( &d->fdd, FDD_READ ); crc_add( f, d );
+	fdd_read_data( d ); crc_add( f, d );
+	fdd_read_data( d ); crc_add( f, d );
 	if( f->crc != 0x000 ) {
 	  f->status_register[2] |= UPD_FDC_ST2_DATA_ERROR;
 	  f->status_register[1] |= UPD_FDC_ST1_CRC_ERROR;
@@ -1158,13 +1194,13 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
 	f->cmd->id != UPD_CMD_INVALID ) {
       f->us = f->data_register[0] & 0x03;
       if( f->current_drive != f->drive[ f->us ] ) {
-        fdd_select( &f->current_drive->fdd, 0 );
+        fdd_select( f->current_drive, 0 );
         f->current_drive = f->drive[ f->us ];
-        fdd_select( &f->current_drive->fdd, 1 );
+        fdd_select( f->current_drive, 1 );
       }
 
       f->hd = ( f->data_register[0] & 0x04 ) >> 2;
-      fdd_set_head( &f->current_drive->fdd, f->hd );
+      fdd_set_head( f->current_drive, f->hd );
 
       /* identify READ_DELETED_DATA/WRITE_DELETED_DATA */
       if( f->cmd->id == UPD_CMD_READ_DATA || 
@@ -1227,9 +1263,9 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
       f->status_register[3] = f->us + ( f->hd << 2 );
       /*???? the plus3 wiring cause that the double side signal is the
          same as the write protect signal... */
-      f->status_register[3] |= d->fdd.wrprot ? UPD_FDC_ST3_WRPROT : 0;
-      f->status_register[3] |= d->fdd.tr00 ? UPD_FDC_ST3_TR00 : 0;
-      f->status_register[3] |= d->fdd.ready ? UPD_FDC_ST3_READY : 0;
+      f->status_register[3] |= d->wrprot ? UPD_FDC_ST3_WRPROT : 0;
+      f->status_register[3] |= d->tr00 ? UPD_FDC_ST3_TR00 : 0;
+      f->status_register[3] |= d->ready ? UPD_FDC_ST3_READY : 0;
       break;
     case UPD_CMD_SENSE_INT:
       for( i = 0; i < 4; i++ ) {
@@ -1240,7 +1276,7 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
             f->status_register[0] |= UPD_FDC_ST0_INT_ABNORM;
 	  else if( f->seek[i] == 6 )
             f->status_register[0] |= UPD_FDC_ST0_INT_READY | UPD_FDC_ST0_NOT_READY;
-	  f->seek[i] = 0;					/* end of seek */
+          f->seek[i] = f->seek_age[i] = 0;			/* end of seek */
     	  f->sense_int_res[0] = f->status_register[0] & 0xfb;	/* return head always 0 (11111011) */
           f->sense_int_res[1] = f->pcn[i];
 	  i = 4;						/* one interrupt o.k. */
@@ -1251,23 +1287,26 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
         f->intrq = UPD_INTRQ_NONE;				/* delete INTRQ state */
       break;
     case UPD_CMD_RECALIBRATE:
+      if( f->main_status & ( 1 << f->us ) ) break;   	/* previous seek in progress? */
       f->rec[f->us] = f->pcn[f->us];			/* save PCN */
       f->pcn[f->us] = 77;
       f->data_register[1] = 0x00;			/* to track0 */
-      f->seek[f->us] = 1;				/* recalibrate started */
-    case UPD_CMD_SEEK:
       f->ncn[f->us] = f->data_register[1];		/* save new cylinder number */
-      f->main_status |= 1 << f->us;			/* selected drive goes busy  */
-      f->seek[f->us]++;					/* = 2 seek started */
-      seek_step( f );
+      f->seek[f->us] = 2;				/* recalibrate started */
+      seek_step( f, 1 );
+      break;
+    case UPD_CMD_SEEK:
+      if( f->main_status & ( 1 << f->us ) ) break;   	/* previous seek in progress? */
+      f->ncn[f->us] = f->data_register[1];		/* save new cylinder number */
+      f->seek[f->us] = 1;				/* seek started */
+      seek_step( f, 1 );
       break;
     case UPD_CMD_READ_ID:
       head_load( f );
       return;
-      break;
     case UPD_CMD_READ_DATA:
     /* Speedlock hack */
-      if( f->speedlock != -1 && !d->fdd.do_read_weak ) {	/* do not conflict with fdd weak read */
+      if( f->speedlock != -1 && !d->do_read_weak ) {	/* do not conflict with fdd weak read */
 	u = ( f->data_register[2] & 0x01 ) + ( f->data_register[1] << 1 ) +
 	    ( f->data_register[3] << 8 );
 	if( f->data_register[3] == f->data_register[5] && u == 0x200 ) {
@@ -1289,17 +1328,15 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
       f->first_rw = 1;		/* always read at least one sector */
       head_load( f );
       return;
-      break;
     case UPD_CMD_READ_DIAG:		/* READ TRACK */
       f->rlen = 0x80 << ( f->data_register[4] > MAX_SIZE_CODE ? MAX_SIZE_CODE : f->data_register[4] );
       if( f->data_register[4] == 0 && f->data_register[7] < 128 )
         f->rlen = f->data_register[7];
       head_load( f );
       return;
-      break;
 
     case UPD_CMD_WRITE_DATA:
-      if( d->fdd.wrprot ) {
+      if( d->wrprot ) {
         f->status_register[1] |= UPD_FDC_ST1_NOT_WRITEABLE;
         f->status_register[0] |= UPD_FDC_ST0_INT_ABNORM;
 	terminated = 1;
@@ -1311,9 +1348,8 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
       f->first_rw = 1;		/* always write at least one sector */
       head_load( f );
       return;
-      break;
     case UPD_CMD_WRITE_ID:			/* FORMAT TRACK */
-      if( d->fdd.wrprot ) {
+      if( d->wrprot ) {
         f->status_register[1] |= UPD_FDC_ST1_NOT_WRITEABLE;
         f->status_register[0] |= UPD_FDC_ST0_INT_ABNORM;
 	terminated = 1;
@@ -1322,7 +1358,6 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
       f->rlen = 0x80 << ( f->data_register[1] > MAX_SIZE_CODE ? MAX_SIZE_CODE : f->data_register[1] ); /* max 8192 byte/sector */
       head_load( f );
       return;
-      break;
     case UPD_CMD_SCAN:			/* & 0x0c >> 2 == 00 - equal, 10 - low, 11 - high */
       f->scan = ( f->command_register & 0x0c ) >> 2 == 0 ? UPD_SCAN_EQ : 
 		( f->command_register & 0x0c ) >> 2 == 0x03 ? UPD_SCAN_HI :
@@ -1331,7 +1366,6 @@ upd_fdc_write_data( upd_fdc *f, libspectrum_byte data )
       f->rlen = 0x80 << ( f->data_register[4] > MAX_SIZE_CODE ? MAX_SIZE_CODE : f->data_register[4] );
       head_load( f );
       return;
-      break;
     }
 
     if( f->cmd->id < UPD_CMD_READ_ID && !terminated ) {	/* we have execution phase */
