@@ -1,7 +1,7 @@
 /* fuller.h: Routines for handling the Fuller Box
-   Copyright (c) 2007-2011 Stuart Brady, Fredrick Meunier, Philip Kendall
+   Copyright (c) 2007-2016 Stuart Brady, Fredrick Meunier, Philip Kendall
 
-   $Id: fuller.h 4640 2012-01-21 13:26:35Z pak21 $
+   $Id: fuller.h 5677 2016-07-09 13:58:02Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,6 @@
 #ifndef FUSE_FULLER_H
 #define FUSE_FULLER_H
 
-void fuller_init( void );
+void fuller_register_startup( void );
 
 #endif				/* #ifndef FUSE_FULLER_H */
