@@ -1,7 +1,7 @@
 /* kempmouse.h: Kempston mouse emulation
-   Copyright (c) 2004-2008 Darren Salt, Fredrick Meunier
+   Copyright (c) 2004-2016 Darren Salt, Fredrick Meunier, Philip Kendall
 
-   $Id: kempmouse.h 4924 2013-05-05 07:40:02Z sbaldovi $
+   $Id: kempmouse.h 5677 2016-07-09 13:58:02Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #ifndef FUSE_KEMPMOUSE_H
 #define FUSE_KEMPMOUSE_H
 
-void kempmouse_init( void );
+void kempmouse_register_startup( void );
 void kempmouse_update( int dx, int dy, int button, int down );
 
 #endif
