@@ -1,7 +1,7 @@
 /* melodik.h: Routines for handling the Melodik interface
-   Copyright (c) 2009-2011 Fredrick Meunier, Philip Kendall
+   Copyright (c) 2009-2016 Fredrick Meunier, Philip Kendall
 
-   $Id: melodik.h 4640 2012-01-21 13:26:35Z pak21 $
+   $Id: melodik.h 5677 2016-07-09 13:58:02Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,6 @@
 #ifndef FUSE_MELODIK_H
 #define FUSE_MELODIK_H
 
-void melodik_init( void );
+void melodik_register_startup( void );
 
 #endif				/* #ifndef FUSE_MELODIK_H */
