@@ -1,7 +1,7 @@
 /* ula.h: ULA routines
-   Copyright (c) 1999-2004 Philip Kendall, Darren Salt
+   Copyright (c) 1999-2016 Philip Kendall, Darren Salt
 
-   $Id: ula.h 4640 2012-01-21 13:26:35Z pak21 $
+   $Id: ula.h 5677 2016-07-09 13:58:02Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ extern libspectrum_byte ula_contention[ ULA_CONTENTION_SIZE ];
 /* And how much when it is inactive */
 extern libspectrum_byte ula_contention_no_mreq[ ULA_CONTENTION_SIZE ];
 
-void ula_init( void );
+void ula_register_startup( void );
 
 libspectrum_byte ula_last_byte( void );
 
