@@ -1,7 +1,7 @@
 /* profile.h: Z80 profiler
-   Copyright (c) 2005 Philip Kendall
+   Copyright (c) 2005-2016 Philip Kendall
 
-   $Id: profile.h 4640 2012-01-21 13:26:35Z pak21 $
+   $Id: profile.h 5677 2016-07-09 13:58:02Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 
 extern int profile_active;
 
-void profile_init( void );
+void profile_register_startup( void );
 void profile_start( void );
 void profile_map( libspectrum_word pc );
 void profile_frame( libspectrum_dword frame_length );
