@@ -1,7 +1,10 @@
 /* debugger.c: the GTK+ debugger
-   Copyright (c) 2002-2012 Philip Kendall
+   Copyright (c) 2002-2015 Philip Kendall
+   Copyright (c) 2013 Sergio Baldoví
+   Copyright (c) 2015 Stuart Brady
+   Copyright (c) 2016 BogDan Vatra
 
-   $Id: debugger.c 4908 2013-03-10 22:13:57Z sbaldovi $
+   $Id: debugger.c 5434 2016-05-01 04:22:45Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +28,7 @@
 
 #include <config.h>
 
+#include <math.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -140,6 +144,13 @@ static void update_events( void );
 static void add_event( gpointer data, gpointer user_data );
 static int deactivate_debugger( void );
 
+static gboolean
+disassembly_key_press( GtkTreeView *list, GdkEventKey *event,
+                       gpointer user_data );
+static gboolean
+disassembly_wheel_scroll( GtkTreeView *list GCC_UNUSED, GdkEvent *event,
+                          gpointer user_data );
+
 static void move_disassembly( GtkAdjustment *adjustment, gpointer user_data );
 
 static void evaluate_command( GtkWidget *widget, gpointer user_data );
@@ -172,6 +183,9 @@ static GtkAdjustment *disassembly_scrollbar_adjustment;
 /* The top line of the current disassembly */
 static libspectrum_word disassembly_top;
 
+/* The next line below the current disassembly */
+static libspectrum_word disassembly_bottom;
+
 /* Have we created the above yet? */
 static int dialog_created = 0;
 
@@ -201,8 +215,6 @@ static GtkActionEntry menu_data[] = {
 
 };
 
-static guint menu_data_count = G_N_ELEMENTS( menu_data );
-
 static GtkToggleActionEntry menu_toggles[] = {
 
   { "VIEW_REGISTERS", NULL, "_Registers", NULL, NULL, G_CALLBACK( toggle_display_registers ), TRUE },
@@ -214,8 +226,6 @@ static GtkToggleActionEntry menu_toggles[] = {
 
 };
 
-static guint menu_toggles_count = G_N_ELEMENTS( menu_toggles );
-
 static const char*
 format_8_bit( void )
 {
@@ -248,6 +258,12 @@ ui_debugger_activate( void )
   return 0;
 }
 
+void
+ui_breakpoints_updated( void )
+{
+  /* TODO: Refresh debugger list here */
+}
+
 static int
 hide_hidden_panes( void )
 {
@@ -407,10 +423,10 @@ create_menu_bar( GtkBox *parent, GtkAccelGroup **accel_group )
 
   /* Load actions */
   menu_action_group = gtk_action_group_new( "DebuggerActionGroup" );
-  gtk_action_group_add_actions( menu_action_group, menu_data, menu_data_count,
-                                NULL );
+  gtk_action_group_add_actions( menu_action_group, menu_data,
+				ARRAY_SIZE( menu_data ), NULL );
   gtk_action_group_add_toggle_actions( menu_action_group, menu_toggles,
-                                       menu_toggles_count, NULL );
+                                       ARRAY_SIZE( menu_toggles ), NULL );
   gtk_ui_manager_insert_action_group( ui_manager_debugger, menu_action_group,
                                       0 );
   g_object_unref( menu_action_group );
@@ -571,8 +587,8 @@ create_breakpoints( GtkBox *parent )
 {
   size_t i;
 
-  gchar *titles[] = { "ID", "Type", "Value", "Ignore", "Life",
-		      "Condition" };
+  static const gchar *const titles[] =
+    { "ID", "Type", "Value", "Ignore", "Life", "Condition" };
 
   breakpoints_model = gtk_list_store_new( BREAKPOINTS_COLUMN_COUNT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING );
 
@@ -592,7 +608,8 @@ create_disassembly( GtkBox *parent, gtkui_font font )
   size_t i;
 
   GtkWidget *scrollbar;
-  const gchar *titles[] = { "Address", "Instruction" };
+  static const gchar *const titles[] =
+    { "Address", "Instruction" };
 
   /* A box to hold the disassembly listing and the scrollbar */
   disassembly_box = gtk_box_new( GTK_ORIENTATION_HORIZONTAL, 0 );
@@ -614,24 +631,30 @@ create_disassembly( GtkBox *parent, gtkui_font font )
 
   /* The disassembly scrollbar */
   disassembly_scrollbar_adjustment = GTK_ADJUSTMENT(
-    gtk_adjustment_new( 0, 0x0000, 0xffff, 0.5, 20, 20 ) );
+    gtk_adjustment_new( 0, 0x0000, 0x10000, 1, 20, 20 ) );
   g_signal_connect( G_OBJECT( disassembly_scrollbar_adjustment ),
 		    "value-changed", G_CALLBACK( move_disassembly ),
 		    NULL );
   scrollbar = gtk_scrollbar_new( GTK_ORIENTATION_VERTICAL,
                                  disassembly_scrollbar_adjustment );
   gtk_box_pack_start( GTK_BOX( disassembly_box ), scrollbar, FALSE, FALSE, 0 );
-/*
-  gtkui_scroll_connect( GTK_CLIST( disassembly ),
-			disassembly_scrollbar_adjustment );
-*/
+
+  /* Scrolling with keys */
+  g_signal_connect( GTK_TREE_VIEW( disassembly ), "key-press-event",
+                    G_CALLBACK( disassembly_key_press ),
+                    disassembly_scrollbar_adjustment );
+
+  /* Scrolling with mouse wheel */
+  g_signal_connect( GTK_TREE_VIEW( disassembly ), "scroll-event",
+                    G_CALLBACK( disassembly_wheel_scroll ),
+                    disassembly_scrollbar_adjustment );
 }
 
 static void
 create_stack_display( GtkBox *parent, gtkui_font font )
 {
   size_t i;
-  const gchar *titles[] = { "Address", "Instruction" };
+  static const gchar *const titles[] = { "Address", "Instruction" };
   
   stack_model =
     gtk_list_store_new( STACK_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING,
@@ -678,7 +701,7 @@ stack_activate( GtkTreeView *tree_view, GtkTreePath *path,
 static void
 create_events( GtkBox *parent )
 {
-  const gchar *titles[] = { "Time", "Type" };
+  static const gchar *const titles[] = { "Time", "Type" };
   size_t i;
 
   events_model =
@@ -819,7 +842,7 @@ ui_debugger_update( void )
   snprintf( &buffer[6], 80, format_8_bit(), ( R & 0x7f ) | ( R7 & 0x80 ) );
   gtk_label_set_text( GTK_LABEL( registers[13] ), buffer );
 
-  snprintf( buffer, 80, "T-states %5d", tstates );
+  snprintf( buffer, 80, "T-states %5d\nHalted %d", tstates, z80.halted );
   gtk_label_set_text( GTK_LABEL( registers[14] ), buffer );
   snprintf( buffer, 80, "  IM %d\nIFF1 %d\nIFF2 %d", IM, IFF1, IFF2 );
   gtk_label_set_text( GTK_LABEL( registers[15] ), buffer );
@@ -1072,6 +1095,8 @@ update_disassembly( void )
 
     address += length;
   }
+
+  disassembly_bottom = address;
 }
 
 static void
@@ -1107,7 +1132,24 @@ int
 ui_debugger_disassemble( libspectrum_word address )
 {
   disassembly_top = address;
+
+  /* Block further events while adjusting scrollbar. */
+  g_signal_handlers_block_by_func( G_OBJECT( disassembly_scrollbar_adjustment ),
+                                   G_CALLBACK( move_disassembly ), NULL );
+
+  /* Note: GtkAdjustment can not cope with "upper bound - page_size" value and
+     higher */
   gtk_adjustment_set_value( disassembly_scrollbar_adjustment, address );
+
+  /* Enable events for scrollbar */
+  g_signal_handlers_unblock_by_func( G_OBJECT(disassembly_scrollbar_adjustment),
+                                     G_CALLBACK( move_disassembly ),  NULL );
+
+  /* And update the disassembly if the debugger is active */
+  if( debugger_active ) {
+    update_disassembly();
+  }
+
   return 0;
 }
 
@@ -1116,18 +1158,22 @@ static void
 move_disassembly( GtkAdjustment *adjustment, gpointer user_data GCC_UNUSED )
 {
   gdouble value;
-  size_t length;
+  int cursor_row;
+  libspectrum_word addresss;
 
+  /* FIXME: Movements are imprecise while dragging the scroll bar */
   value = gtk_adjustment_get_value( adjustment );
 
-  /* disassembly_top < value < disassembly_top + 1 => 'down' button pressed
+  cursor_row = gtkui_list_get_cursor( GTK_TREE_VIEW( disassembly ) );
+
+  /* disassembly_top < value <= disassembly_top + 1 => 'down' button pressed
      Move the disassembly on by one instruction */
-  if( value > disassembly_top && value - disassembly_top < 1 ) {
+  if( value > disassembly_top && value - disassembly_top <= 1 ) {
 
-    debugger_disassemble( NULL, 0, &length, disassembly_top );
-    ui_debugger_disassemble( disassembly_top + length );
+    addresss = debugger_search_instruction( disassembly_top, 1 );
+    ui_debugger_disassemble( addresss );
 
-  /* disassembly_top - 1 < value < disassembly_top => 'up' button pressed
+  /* disassembly_top - 1 <= value < disassembly_top => 'up' button pressed
      
      The desired state after this is for the current top instruction
      to be the second instruction shown in the disassembly.
@@ -1148,28 +1194,151 @@ move_disassembly( GtkAdjustment *adjustment, gpointer user_data GCC_UNUSED )
      second. In this case, just move back a byte.
 
   */
-  } else if( value < disassembly_top && disassembly_top - value < 1 ) {
+  } else if( value < disassembly_top && disassembly_top - value <= 1 ) {
 
-    size_t i, longest = 1;
-
-    for( i = 1; i <= 8; i++ ) {
-
-      debugger_disassemble( NULL, 0, &length, disassembly_top - i );
-      if( length == i ) longest = i;
-
-    }
-
-    ui_debugger_disassemble( disassembly_top - longest );
+    addresss = debugger_search_instruction( disassembly_top, -1 );
+    ui_debugger_disassemble( addresss );
 
   /* Anything else, just set disassembly_top to that value */
-  } else {
+  } else if( value != disassembly_top ) {
 
     ui_debugger_disassemble( value );
 
   }
 
-  /* And update the disassembly if the debugger is active */
-  if( debugger_active ) update_disassembly();
+  /* Mark selected row */
+  gtkui_list_set_cursor( GTK_TREE_VIEW( disassembly ), cursor_row );
+}
+
+static gboolean
+disassembly_key_press( GtkTreeView *list, GdkEventKey *event,
+                       gpointer user_data )
+{
+  GtkAdjustment *adjustment = user_data;
+  gdouble page_size, page_increment;
+  int cursor_row;
+  libspectrum_word initial_top, addresss;
+
+  initial_top = disassembly_top;
+  page_size = gtk_adjustment_get_page_size( adjustment );
+  page_increment = gtk_adjustment_get_page_increment( adjustment );
+
+  /* Get selected row */
+  cursor_row = gtkui_list_get_cursor( list );
+
+  switch( event->keyval ) {
+
+  case GDK_KEY_Down:
+    if( cursor_row == page_size - 1 ) {
+      addresss = debugger_search_instruction( disassembly_top, 1 );
+      ui_debugger_disassemble( addresss );
+    }
+    break;
+
+  case GDK_KEY_Up:
+    if( cursor_row == 0 ) {
+      addresss = debugger_search_instruction( disassembly_top, -1 );
+      ui_debugger_disassemble( addresss );
+    }
+    break;
+
+  case GDK_KEY_Page_Down:
+    ui_debugger_disassemble( disassembly_bottom );
+    break;
+
+  case GDK_KEY_Page_Up:
+    addresss = debugger_search_instruction( disassembly_top, -page_increment );
+    ui_debugger_disassemble( addresss );
+    break;
+
+  case GDK_KEY_Home:
+    cursor_row = 0;
+    ui_debugger_disassemble( 0x0000 );
+    break;
+
+  case GDK_KEY_End:
+    cursor_row = page_size - 1;
+    addresss = debugger_search_instruction( 0x0000, -page_size );
+    ui_debugger_disassemble( addresss );
+    break;
+
+  default:
+    return FALSE;
+  }
+
+  if( initial_top != disassembly_top ) {
+    update_disassembly();
+
+    /* Mark selected row */
+    gtkui_list_set_cursor( list, cursor_row );
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+/* Called when the wheel mouse is moved on the list (not on the scrollbar) */
+static gboolean
+disassembly_wheel_scroll( GtkTreeView *list GCC_UNUSED, GdkEvent *event,
+                          gpointer user_data )
+{
+  libspectrum_word initial_top, addresss;
+  int cursor_row;
+
+  initial_top = disassembly_top;
+
+  /* Get selected row */
+  cursor_row = gtkui_list_get_cursor( list );
+
+  switch( event->scroll.direction ) {
+  case GDK_SCROLL_UP:
+    addresss = debugger_search_instruction( disassembly_top, -1 );
+    ui_debugger_disassemble( addresss );
+    break;
+
+  case GDK_SCROLL_DOWN:
+    addresss = debugger_search_instruction( disassembly_top, 1 );
+    ui_debugger_disassemble( addresss );
+    break;
+
+#if GTK_CHECK_VERSION( 3, 4, 0 )
+
+  case GDK_SCROLL_SMOOTH:
+    {
+      GtkAdjustment *adjustment = user_data;
+      static gdouble total_dy = 0;
+      gdouble dx, dy, page_size;
+      int delta;
+
+      if( gdk_event_get_scroll_deltas( event, &dx, &dy ) ) {
+        /* Calculate number of instructions to jump */
+        total_dy += dy;
+        page_size = gtk_adjustment_get_page_size( adjustment );
+        delta = total_dy * pow( page_size, 2.0 / 3.0 );
+
+        /* Is movement significative? */
+        if( delta ) {
+          addresss = debugger_search_instruction( disassembly_top, delta );
+          ui_debugger_disassemble( addresss );
+          total_dy = 0;
+        }
+      }
+      break;
+    }
+
+#endif
+
+  default:
+    return FALSE;
+  }
+
+  if( initial_top != disassembly_top ) {
+    /* Mark selected row */
+    gtkui_list_set_cursor( list, cursor_row );
+    return TRUE;
+  }
+
+  return FALSE;
 }
 
 /* Evaluate the command currently in the entry box */
