@@ -1,7 +1,7 @@
 /* gtkinternals.h: stuff internal to the GTK+ UI
-   Copyright (c) 2003-2005 Philip Kendall
+   Copyright (c) 2003-2015 Philip Kendall
 
-   $Id: gtkinternals.h 4962 2013-05-19 05:25:15Z sbaldovi $
+   $Id: gtkinternals.h 5510 2016-05-22 08:37:30Z sbaldovi $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,6 +36,8 @@
 /* The colour palette in use */
 extern libspectrum_dword gtkdisplay_colours[ 16 ];
 
+void gtkdisplay_update_geometry( void );
+
 /*
  * Keyboard routines (gtkkeyboard.c)
  */
@@ -76,7 +78,7 @@ GtkAccelGroup* gtkstock_add_accel_group( GtkWidget *widget );
  * For either, GDK_KEY_VoidSymbol means "no accel key".
  */
 typedef struct gtkstock_button {
-  gchar *label;
+  const gchar *label;
   GCallback action;		/* "clicked" func; data is actiondata. */
   gpointer actiondata;
   GCallback destroy;	/* "clicked" func; data is parent widget */
@@ -89,9 +91,7 @@ typedef struct gtkstock_button {
 /* GTK1: create a simple button with the given label.
  *   "gtk-" prefixes are stripped and are used to select default accel keys.
  * GTK2: chooses between stock and normal based on "gtk-" prefix.
- *
- * Some stock labels are defined below for use with GTK1: their names are the
- * same as in GTK2, but the strings are different.
+ * GTK3: GtkStock has been deprecated, we should not use "gtk-" prefix.
  *
  * If the target widget is a GtkDialog, then created buttons are put in its
  * action area.
@@ -125,6 +125,8 @@ int gtkui_get_monospaced_font( gtkui_font *font );
 void gtkui_free_font( gtkui_font font );
 void gtkui_set_font( GtkWidget *widget, gtkui_font font );
 
+int gtkui_menubar_get_height( void );
+
 /*
  * The menu data (menu_data.c)
  */
@@ -151,15 +153,19 @@ extern const char *gtkpixmap_mouse_active[];
  * Statusbar routines (statusbar.c)
  */
 
+
 int gtkstatusbar_create( GtkBox *parent );
+int gtkstatusbar_get_height( void );
 int gtkstatusbar_set_visibility( int visible );
 void gtkstatusbar_update_machine( const char *name );
 
 /*
- * Scrolling for GtkCList widgets
+ * Routines for list widgets
  */
 
 void gtkui_scroll_connect( GtkTreeView *list, GtkAdjustment *adj );
+void gtkui_list_set_cursor( GtkTreeView *list, int row );
+int gtkui_list_get_cursor( GtkTreeView *list );
 
 /*
  * Dialog box reset
