@@ -1,7 +1,7 @@
 /* pokemem.c: GTK+ interface that handles pok files
-   Copyright (c) 2011 Philip Kendall, Sergio Baldoví
+   Copyright (c) 2011-2015 Philip Kendall, Sergio Baldoví
 
-   $Id: pokemem.c 4962 2013-05-19 05:25:15Z sbaldovi $
+   $Id: pokemem.c 5434 2016-05-01 04:22:45Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -157,7 +157,7 @@ create_dialog( void )
 
   /* Create Add button for custom pokes */
   static const gtkstock_button
-    add  = { "Add", G_CALLBACK( pokemem_add_custom_poke ), NULL, NULL,
+    add  = { "_Add", G_CALLBACK( pokemem_add_custom_poke ), NULL, NULL,
              0, 0, 0, 0 };
   gtkstock_create_button( GTK_WIDGET( hbox ), accel_group, &add );
 
@@ -202,7 +202,10 @@ create_and_fill_treeview( void )
   GtkTreeModel *model;
 
   poke_list = gtk_tree_view_new();
+
+#if !GTK_CHECK_VERSION( 3, 0, 0 )
   gtk_tree_view_set_rules_hint( GTK_TREE_VIEW( poke_list ), TRUE );
+#endif
 
   store = gtk_list_store_new( NUM_COLS, G_TYPE_BOOLEAN, G_TYPE_STRING,
                               G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_BOOLEAN,
