@@ -1,7 +1,8 @@
 /* ui.h: General UI event handling routines
-   Copyright (c) 2000-2004 Philip Kendall
+   Copyright (c) 2000-2015 Philip Kendall
+   Copyright (c) 2016 BogDan Vatra
 
-   $Id: ui.h 4664 2012-02-12 11:51:01Z fredm $
+   $Id: ui.h 5560 2016-05-31 11:15:34Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,9 +38,11 @@
 #include "compat.h"
 #include "machines/specplus3.h"
 #include "peripherals/disk/beta.h"
+#include "peripherals/disk/didaktik.h"
 #include "peripherals/disk/disciple.h"
 #include "peripherals/disk/opus.h"
 #include "peripherals/disk/plusd.h"
+#include "svg.h"
 #include "ui/scaler/scaler.h"
 
 /* The various severities of error level, increasing downwards */
@@ -60,8 +63,10 @@ int ui_end(void);
 int ui_error( ui_error_level severity, const char *format, ... )
      GCC_PRINTF( 2, 3 );
 libspectrum_error ui_libspectrum_error( libspectrum_error error,
-					const char *format, va_list ap );
-int ui_verror( ui_error_level severity, const char *format, va_list ap );
+					const char *format, va_list ap )
+     GCC_PRINTF( 2, 0 );
+int ui_verror( ui_error_level severity, const char *format, va_list ap )
+     GCC_PRINTF( 2, 0 );
 int ui_error_specific( ui_error_level severity, const char *message );
 void ui_error_frame( void );
 
@@ -70,6 +75,7 @@ int ui_debugger_activate( void );
 int ui_debugger_deactivate( int interruptable );
 int ui_debugger_update( void );
 int ui_debugger_disassemble( libspectrum_word address );
+void ui_breakpoints_updated();
 
 /* Reset anything in the UI which needs to be reset on machine selection */
 int ui_widgets_reset( void );
@@ -115,12 +121,6 @@ int ui_mouse_release( int suspend ); /* UI: ungrab, return 0 if done */
 /* Write the current tape out */
 int ui_tape_write( void );
 
-/* Write a +3, Beta or +D disk out */
-int ui_plus3_disk_write( specplus3_drive_number which, int saveas );
-int ui_beta_disk_write( beta_drive_number which, int saveas );
-int ui_opus_disk_write( opus_drive_number which, int saveas );
-int ui_plusd_disk_write( plusd_drive_number which, int saveas );
-int ui_disciple_disk_write( disciple_drive_number which, int saveas );
 int ui_mdr_write( int which, int saveas );
 
 /* Get a rollback point from the given list */
@@ -130,9 +130,12 @@ int ui_get_rollback_point( GSList *points );
 
 typedef enum ui_menu_item {
 
+  UI_MENU_ITEM_INVALID = 0,
+  UI_MENU_ITEM_FILE_SVG_CAPTURE,
   UI_MENU_ITEM_FILE_MOVIE_RECORDING,
   UI_MENU_ITEM_FILE_MOVIE_PAUSE,
   UI_MENU_ITEM_MACHINE_PROFILER,
+  UI_MENU_ITEM_MACHINE_DIDAKTIK80_SNAP,
   UI_MENU_ITEM_MEDIA_CARTRIDGE,
   UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK,
   UI_MENU_ITEM_MEDIA_CARTRIDGE_DOCK_EJECT,
@@ -193,6 +196,15 @@ typedef enum ui_menu_item {
   UI_MENU_ITEM_MEDIA_DISK_PLUSD_2_EJECT,
   UI_MENU_ITEM_MEDIA_DISK_PLUSD_2_FLIP_SET,
   UI_MENU_ITEM_MEDIA_DISK_PLUSD_2_WP_SET,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK_A,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK_A_EJECT,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK_A_FLIP_SET,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK_A_WP_SET,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK_B,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK_B_EJECT,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK_B_FLIP_SET,
+  UI_MENU_ITEM_MEDIA_DISK_DIDAKTIK_B_WP_SET,
   UI_MENU_ITEM_MEDIA_DISK_DISCIPLE,
   UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1,
   UI_MENU_ITEM_MEDIA_DISK_DISCIPLE_1_EJECT,
