@@ -1,7 +1,9 @@
 /* win32ui.c: Win32 routines for dealing with the user interface
-   Copyright (c) 2003-2007 Marek Januszewski, Philip Kendall, Stuart Brady
+   Copyright (c) 2003-2015 Marek Januszewski, Philip Kendall, Stuart Brady
+   Copyright (c) 2015 Kirben
+   Copyright (c) 2016 lordhoto
 
-   $Id: win32ui.c 4968 2013-05-19 16:11:17Z zubzero $
+   $Id: win32ui.c 5434 2016-05-01 04:22:45Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,7 +33,6 @@
 #include "fuse.h"
 #include "keyboard.h"
 #include "menu.h"
-#include "menu_data.h"
 #include "peripherals/joystick.h"
 #include "psg.h"
 #include "rzx.h"
@@ -42,6 +43,7 @@
 #include "tape.h"
 #include "timer/timer.h"
 #include "ui/ui.h"
+#include "ui/win32/menu_data.h"
 #include "utils.h"
 #include "win32internals.h"
 #include "win32joystick.h"
@@ -77,7 +79,7 @@ typedef struct win32ui_select_info {
   int length;
   int selected;
   const char **labels;
-  TCHAR *dialog_title;
+  LPCTSTR dialog_title;
 
 } win32ui_select_info;
 
@@ -93,8 +95,6 @@ static BOOL win32ui_window_resizing( HWND hWnd, WPARAM wParam, LPARAM lParam );
 static int
 selector_dialog( win32ui_select_info *items );
 
-#define DIM(X) sizeof((X)) / sizeof((X)[0])
-
 static void
 handle_drop( HDROP hDrop )
 {
@@ -303,7 +303,21 @@ WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
   fuse_nCmdShow = nCmdShow;
   fuse_hPrevInstance = hPrevInstance;
 
-  return fuse_main(__argc, __argv);
+/* HACK: __argc, __argv are broken and return zero when using mingwrt 4.0+
+   on MinGW.
+   HACK: MinGW-w64 based toolchains neither feature _argc nor _argv. The 32 bit
+   incarnation only defines __MINGW32__. This leads to build breakage due to
+   missing declarations. Luckily MinGW-w64 based toolchains define
+   __MINGW64_VERSION_foo macros inside _mingw.h, which is included from all
+   system headers. Thus we abuse that to detect them.
+*/
+#if defined( __GNUC__ ) && defined( __MINGW32__ ) \
+                        && !defined( __MINGW64_VERSION_MAJOR )
+  return fuse_main( _argc, _argv );
+#else
+  return fuse_main( __argc, __argv );
+#endif
+
   /* FIXME: how do deal with returning wParam */
 }
 
@@ -389,7 +403,9 @@ win32ui_make_menu( void )
   ui_menu_activate( UI_MENU_ITEM_RECORDING, 0 );
   ui_menu_activate( UI_MENU_ITEM_RECORDING_ROLLBACK, 0 );
   ui_menu_activate( UI_MENU_ITEM_TAPE_RECORDING, 0 );
-
+#ifdef HAVE_LIB_XML2
+  ui_menu_activate( UI_MENU_ITEM_FILE_SVG_CAPTURE, 0 );
+#endif
   return FALSE;
 }
 
@@ -694,7 +710,7 @@ ui_confirm_joystick_t
 ui_confirm_joystick( libspectrum_joystick libspectrum_type, int inputs )
 {
   win32ui_select_info items;
-  char title[ 80 ];
+  TCHAR title[ 80 ];
   int i, selection;
   int selected_joystick;
 
@@ -711,7 +727,8 @@ ui_confirm_joystick( libspectrum_joystick libspectrum_type, int inputs )
   fuse_emulation_pause();
 
   /* Populate win32ui_select_info */
-  _sntprintf( title, sizeof( title ), "Fuse - Configure %s Joystick",
+  /* FIXME: libspectrum_joystick_name is not unicode compliant */
+  _sntprintf( title, ARRAY_SIZE( title ), _T( "Fuse - Configure %s Joystick" ),
 	    libspectrum_joystick_name( libspectrum_type ) );
   items.dialog_title = title;
   items.length = JOYSTICK_CONN_COUNT; 
@@ -750,7 +767,7 @@ win32ui_get_monospaced_font( HFONT *font )
 
     *font = CreateFont( font_height, 0, 0, 0, 400, FALSE, FALSE, FALSE, 0,
                         400, 2, 1, 1, TEXT( "Courier New" ) );
-    if( !font ) {
+    if( *font == NULL ) {
       ui_error( UI_ERROR_ERROR, "couldn't find a monospaced font" );
       return 1;
     }
@@ -764,7 +781,7 @@ win32ui_get_monospaced_font( HFONT *font )
 }
 
 int
-window_recommended_width( HWND hwndDlg, TCHAR *title )
+window_recommended_width( HWND hwndDlg, LPCTSTR title )
 {
   HDC dc;
   SIZE sz;
@@ -940,7 +957,7 @@ selector_dialog_proc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
       if ( HIWORD( wParam ) != BN_CLICKED ) break;
 
       /* service OK and Cancel buttons */
-      switch LOWORD( wParam )
+      switch( LOWORD( wParam ) )
       {
         case IDOK:
         {
@@ -1020,6 +1037,9 @@ win32ui_window_resize( HWND hWnd, WPARAM wParam, LPARAM lParam )
     if( !size_paused ) {
       size_paused = 1;
       fuse_emulation_pause();
+
+      /* Process UI events until the window is restored */
+      win32ui_process_messages( 0 );
     }
   } else {
     win32display_drawing_area_resize( LOWORD( lParam ), HIWORD( lParam ), 1 );
@@ -1029,6 +1049,9 @@ win32ui_window_resize( HWND hWnd, WPARAM wParam, LPARAM lParam )
     win32statusbar_resize( hWnd, wParam, lParam );
 
     if( size_paused ) {
+      timer_estimate_reset();
+      PostMessage( fuse_hWnd, WM_USER_EXIT_PROCESS_MESSAGES, 0, 0 );
+
       size_paused = 0;
       fuse_emulation_unpause();
     }
@@ -1172,7 +1195,7 @@ win32ui_process_messages( int process_queue_once )
       /* FIXME: rethink this loop, IsDialogMessage in particular */
       processMsg = TRUE;
 
-      for( i = 0; processMsg && i < DIM( hModelessDlgs ); i++) {
+      for( i = 0; processMsg && i < ARRAY_SIZE( hModelessDlgs ); i++) {
         if( IsDialogMessage( hModelessDlgs[i], &msg ) ) processMsg = FALSE;
       }
 
