@@ -1,7 +1,7 @@
 /* coretest.c: Test program for Fuse's Z80 core
-   Copyright (c) 2003-2013 Philip Kendall
+   Copyright (c) 2003-2015 Philip Kendall
 
-   $Id: coretest.c 4905 2013-03-08 20:21:40Z pak21 $
+   $Id: coretest.c 5677 2016-07-09 13:58:02Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,7 +30,24 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "fuse.h"
+#include "peripherals/disk/beta.h"
+#include "peripherals/disk/didaktik.h"
+#include "peripherals/disk/disciple.h"
+#include "peripherals/disk/opus.h"
+#include "peripherals/disk/plusd.h"
+#include "peripherals/ide/divide.h"
+#include "peripherals/if1.h"
+#include "peripherals/spectranet.h"
+#include "peripherals/ula.h"
+#include "peripherals/usource.h"
+#include "profile.h"
+#include "rzx.h"
+#include "slt.h"
+#include "tape.h"
+
 #include "event.h"
+#include "infrastructure/startup_manager.h"
 #include "module.h"
 #include "spectrum.h"
 #include "ui/ui.h"
@@ -77,7 +94,7 @@ main( int argc, char **argv )
   if( init_dummies() ) return 1;
 
   /* Initialise the tables used by the Z80 core */
-  z80_init();
+  z80_init( NULL );
 
   f = fopen( testsfile, "r" );
   if( !f ) {
@@ -408,6 +425,13 @@ debugger_check( debugger_breakpoint_type type GCC_UNUSED, libspectrum_dword valu
   abort();
 }
 
+void debugger_system_variable_register(
+  const char *type, const char *detail,
+  debugger_get_system_variable_fn_t get,
+  debugger_set_system_variable_fn_t set )
+{
+}
+
 int
 debugger_trap( void )
 {
@@ -477,6 +501,31 @@ disciple_page( void )
   abort();
 }
 
+int didaktik80_available = 0;
+int didaktik80_active = 0;
+int didaktik80_snap = 0;
+
+void
+didaktik80_page( void )
+{
+  abort();
+}
+
+void
+didaktik80_unpage( void )
+{
+  abort();
+}
+
+int usource_available = 0;
+int usource_active = 0;
+
+void
+usource_toggle( void )
+{
+  abort();
+}
+
 void
 if1_page( void )
 {
@@ -498,7 +547,7 @@ divide_set_automap( int state GCC_UNUSED )
 int spectranet_available = 0;
 
 void
-spectranet_page( void )
+spectranet_page( int via_io GCC_UNUSED )
 {
   abort();
 }
@@ -526,6 +575,22 @@ spectranet_nmi_flipflop( void )
   return 0;
 }
 
+void
+startup_manager_register( startup_manager_module module,
+  startup_manager_module *dependencies, size_t dependency_count,
+  startup_manager_init_fn init_fn, void *init_context,
+  startup_manager_end_fn end_fn )
+{
+}
+
+int svg_capture_active = 0;     /* SVG capture enabled? */
+
+void
+svg_capture( void )
+{
+  abort();
+}
+
 int
 rzx_frame( void )
 {
@@ -551,6 +616,11 @@ module_register( module_info_t *module GCC_UNUSED )
   return 0;
 }
 
+void
+z80_debugger_variables_init( void )
+{
+}
+
 fuse_machine_info *machine_current;
 static fuse_machine_info dummy_machine;
 
@@ -581,6 +651,7 @@ init_dummies( void )
   scld_last_dec.name.intdisable = 0;
   settings_current.slt_traps = 0;
   settings_current.divide_enabled = 0;
+  settings_current.z80_is_cmos = 0;
   beta_pc_mask = 0xfe00;
   beta_pc_value = 0x3c00;
   spectranet_programmable_trap_active = 0;
